@@ -14,6 +14,7 @@ from datasets.download.download_config import DownloadConfig
 from datasets.download.download_manager import DownloadManager
 from datasets.download.mock_download_manager import MockDownloadManager
 from datasets.load import dataset_module_factory, import_main_class
+from datasets.utils.deprecation_utils import deprecated
 from datasets.utils.logging import get_logger, set_verbosity_warning
 from datasets.utils.py_utils import map_nested
 
@@ -23,7 +24,7 @@ logger = get_logger(__name__)
 DEFAULT_ENCODING = "utf-8"
 
 
-def test_command_factory(args):
+def dummy_data_command_factory(args):
     return DummyDataCommand(
         args.path_to_dataset,
         args.auto_generate,
@@ -212,6 +213,9 @@ class DummyDataGeneratorDownloadManager(DownloadManager):
         shutil.rmtree(base_name)
 
 
+@deprecated(
+    "The `datasets` repository does not host the dataset scripts anymore. Therefore, dummy data is no longer needed to test their loading with CI."
+)
 class DummyDataCommand(BaseDatasetsCLICommand):
     @staticmethod
     def register_subcommand(parser: ArgumentParser):
@@ -256,7 +260,7 @@ class DummyDataCommand(BaseDatasetsCLICommand):
             help=f"Encoding to use when auto-generating dummy data. Defaults to {DEFAULT_ENCODING}",
         )
         test_parser.add_argument("path_to_dataset", type=str, help="Path to the dataset (example: ./datasets/squad)")
-        test_parser.set_defaults(func=test_command_factory)
+        test_parser.set_defaults(func=dummy_data_command_factory)
 
     def __init__(
         self,
