@@ -9,7 +9,7 @@ import xml.dom.minidom
 from asyncio import TimeoutError
 from io import BytesIO
 from itertools import chain
-from pathlib import Path, PurePath, PurePosixPath
+from pathlib import Path, PurePosixPath
 from typing import Callable, Generator, Iterable, List, Optional, Tuple, Union
 from xml.etree import ElementTree as ET
 
@@ -53,8 +53,6 @@ COMPRESSION_EXTENSION_TO_PROTOCOL = {
     **{fs_class.extension.lstrip("."): fs_class.protocol for fs_class in COMPRESSION_FILESYSTEMS},
     # archive compression
     "zip": "zip",
-    "tar": "tar",
-    "tgz": "tar",
 }
 SINGLE_FILE_COMPRESSION_PROTOCOLS = {fs_class.protocol for fs_class in COMPRESSION_FILESYSTEMS}
 SINGLE_SLASH_AFTER_PROTOCOL_PATTERN = re.compile(r"(?<!:):/")
@@ -104,10 +102,10 @@ def xjoin(a, *p):
     """
     a, *b = str(a).split("::")
     if is_local_path(a):
-        a = Path(a, *p).as_posix()
+        return os.path.join(a, *p)
     else:
         a = posixpath.join(a, *p)
-    return "::".join([a] + b)
+        return "::".join([a] + b)
 
 
 def xdirname(a):
@@ -225,10 +223,12 @@ def xisfile(path, use_auth_token: Optional[Union[str, bool]] = None) -> bool:
     """Extend `os.path.isfile` function to support remote files.
 
     Args:
-        path (:obj:`str`): URL path.
+        path (`str`): URL path.
+        use_auth_token (`bool` or `str`, *optional*): Whether to use token or token to authenticate on the
+            Hugging Face Hub for private remote files.
 
     Returns:
-        :obj:`bool`
+        `bool`
     """
     main_hop, *rest_hops = str(path).split("::")
     if is_local_path(main_hop):
@@ -252,10 +252,12 @@ def xgetsize(path, use_auth_token: Optional[Union[str, bool]] = None) -> int:
     """Extend `os.path.getsize` function to support remote files.
 
     Args:
-        path (:obj:`str`): URL path.
+        path (`str`): URL path.
+        use_auth_token (`bool` or `str`, *optional*): Whether to use token or token to authenticate on the
+            Hugging Face Hub for private remote files.
 
     Returns:
-        :obj:`int`, optional
+        `int`: optional
     """
     main_hop, *rest_hops = str(path).split("::")
     if is_local_path(main_hop):
@@ -284,10 +286,12 @@ def xisdir(path, use_auth_token: Optional[Union[str, bool]] = None) -> bool:
     """Extend `os.path.isdir` function to support remote files.
 
     Args:
-        path (:obj:`str`): URL path.
+        path (`str`): URL path.
+        use_auth_token (`bool` or `str`, *optional*): Whether to use token or token to authenticate on the
+            Hugging Face Hub for private remote files.
 
     Returns:
-        :obj:`bool`
+        `bool`
     """
     main_hop, *rest_hops = str(path).split("::")
     if is_local_path(main_hop):
@@ -313,11 +317,11 @@ def xrelpath(path, start=None):
     """Extend `os.path.relpath` function to support remote files.
 
     Args:
-        path (:obj:`str`): URL path.
-        start (:obj:`str`): Start URL directory path.
+        path (`str`): URL path.
+        start (`str`): Start URL directory path.
 
     Returns:
-        :obj:`str`
+        `str`
     """
     main_hop, *rest_hops = str(path).split("::")
     if is_local_path(main_hop):
@@ -326,21 +330,6 @@ def xrelpath(path, start=None):
         return posixpath.relpath(main_hop, start=str(start).split("::")[0]) if start else os.path.relpath(main_hop)
 
 
-def _as_posix(path: Path):
-    """Extend :meth:`pathlib.PurePath.as_posix` to fix missing slashes after protocol.
-
-    Args:
-        path (:obj:`~pathlib.Path`): Calling Path instance.
-
-    Returns:
-        obj:`str`
-    """
-    path_as_posix = path.as_posix()
-    path_as_posix = SINGLE_SLASH_AFTER_PROTOCOL_PATTERN.sub("://", path_as_posix)
-    path_as_posix += "//" if path_as_posix.endswith(":") else ""  # Add slashes to root of the protocol
-    return path_as_posix
-
-
 def _add_retries_to_file_obj_read_method(file_obj):
     read = file_obj.read
     max_retries = config.STREAMING_READ_MAX_RETRIES
@@ -386,9 +375,15 @@ def _get_extraction_protocol(urlpath: str, use_auth_token: Optional[Union[str, b
         extension = extension.split(symb)[0]
     if extension in BASE_KNOWN_EXTENSIONS:
         return None
-    elif path.endswith(".tar.gz") or path.endswith(".tgz"):
+    elif extension in ["tgz", "tar"] or path.endswith(".tar.gz"):
         raise NotImplementedError(
-            f"Extraction protocol for TAR archives like '{urlpath}' is not implemented in streaming mode. Please use `dl_manager.iter_archive` instead."
+            f"Extraction protocol for TAR archives like '{urlpath}' is not implemented in streaming mode. "
+            f"Please use `dl_manager.iter_archive` instead.\n\n"
+            f"Example usage:\n\n"
+            f"\turl = dl_manager.download(url)\n"
+            f"\ttar_archive_iterator = dl_manager.iter_archive(url)\n\n"
+            f"\tfor filename, file in tar_archive_iterator:\n"
+            f"\t\t..."
         )
     elif extension in COMPRESSION_EXTENSION_TO_PROTOCOL:
         return COMPRESSION_EXTENSION_TO_PROTOCOL[extension]
@@ -427,20 +422,30 @@ def _prepare_http_url_kwargs(url: str, use_auth_token: Optional[Union[str, bool]
 
 
 def xopen(file: str, mode="r", *args, use_auth_token: Optional[Union[str, bool]] = None, **kwargs):
-    """
-    This function extends the builtin `open` function to support remote files using fsspec.
+    """Extend `open` function to support remote files using `fsspec`.
 
     It also has a retry mechanism in case connection fails.
-    The args and kwargs are passed to fsspec.open, except `use_auth_token` which is used for queries to private repos on huggingface.co
+    The `args` and `kwargs` are passed to `fsspec.open`, except `use_auth_token` which is used for queries to private repos on huggingface.co
+
+    Args:
+        file (`str`): Path name of the file to be opened.
+        mode (`str`, *optional*, default "r"): Mode in which the file is opened.
+        *args: Arguments to be passed to `fsspec.open`.
+        use_auth_token (`bool` or `str`, *optional*): Whether to use token or token to authenticate on the
+            Hugging Face Hub for private remote files.
+        **kwargs: Keyword arguments to be passed to `fsspec.open`.
+
+    Returns:
+        file object
     """
-    # required for `xopen(str(Path(...)))` to work
-    file = _as_posix(PurePath(file))
-    main_hop, *rest_hops = file.split("::")
+    # This works as well for `xopen(str(Path(...)))`
+    file_str = _as_str(file)
+    main_hop, *rest_hops = file_str.split("::")
     if is_local_path(main_hop):
-        return open(file, mode, *args, **kwargs)
+        return open(main_hop, mode, *args, **kwargs)
     # add headers and cookies for authentication on the HF Hub and for Google Drive
     if not rest_hops and (main_hop.startswith("http://") or main_hop.startswith("https://")):
-        file, new_kwargs = _prepare_http_url_kwargs(file, use_auth_token=use_auth_token)
+        file, new_kwargs = _prepare_http_url_kwargs(file_str, use_auth_token=use_auth_token)
     elif rest_hops and (rest_hops[0].startswith("http://") or rest_hops[0].startswith("https://")):
         url = rest_hops[0]
         url, http_kwargs = _prepare_http_url_kwargs(url, use_auth_token=use_auth_token)
@@ -467,12 +472,14 @@ def xlistdir(path: str, use_auth_token: Optional[Union[str, bool]] = None) -> Li
     """Extend `os.listdir` function to support remote files.
 
     Args:
-        path (:obj:`str`): URL path.
+        path (`str`): URL path.
+        use_auth_token (`bool` or `str`, *optional*): Whether to use token or token to authenticate on the
+            Hugging Face Hub for private remote files.
 
     Returns:
-        :obj:`list` of :obj:`str`
+        `list` of `str`
     """
-    main_hop, *rest_hops = str(path).split("::")
+    main_hop, *rest_hops = _as_str(path).split("::")
     if is_local_path(main_hop):
         return os.listdir(path)
     else:
@@ -498,14 +505,16 @@ def xglob(urlpath, *, recursive=False, use_auth_token: Optional[Union[str, bool]
     """Extend `glob.glob` function to support remote files.
 
     Args:
-        urlpath (:obj:`str`): URL path with shell-style wildcard patterns.
-        recursive (:obj:`bool`, default `False`): Whether to match the "**" pattern recursively to zero or more
+        urlpath (`str`): URL path with shell-style wildcard patterns.
+        recursive (`bool`, default `False`): Whether to match the "**" pattern recursively to zero or more
             directories or subdirectories.
+        use_auth_token (`bool` or `str`, *optional*): Whether to use token or token to authenticate on the
+            Hugging Face Hub for private remote files.
 
     Returns:
-        :obj:`list` of :obj:`str`
+        `list` of `str`
     """
-    main_hop, *rest_hops = str(urlpath).split("::")
+    main_hop, *rest_hops = _as_str(urlpath).split("::")
     if is_local_path(main_hop):
         return glob.glob(main_hop, recursive=recursive)
     else:
@@ -526,23 +535,26 @@ def xglob(urlpath, *, recursive=False, use_auth_token: Optional[Union[str, bool]
         # - If there is "**" in the pattern, `fs.glob` must be called anyway.
         inner_path = main_hop.split("://")[1]
         globbed_paths = fs.glob(inner_path)
-        return ["::".join([f"{fs.protocol}://{globbed_path}"] + rest_hops) for globbed_path in globbed_paths]
+        protocol = fs.protocol if isinstance(fs.protocol, str) else fs.protocol[-1]
+        return ["::".join([f"{protocol}://{globbed_path}"] + rest_hops) for globbed_path in globbed_paths]
 
 
-def xwalk(urlpath, use_auth_token: Optional[Union[str, bool]] = None):
+def xwalk(urlpath, use_auth_token: Optional[Union[str, bool]] = None, **kwargs):
     """Extend `os.walk` function to support remote files.
 
     Args:
-        urlpath (:obj:`str`): URL root path.
-        use_auth_token (:obj:`bool` or :obj:`str`, optional): Whether to use token or token to authenticate on the
+        urlpath (`str`): URL root path.
+        use_auth_token (`bool` or `str`, *optional*): Whether to use token or token to authenticate on the
             Hugging Face Hub for private remote files.
+        **kwargs: Additional keyword arguments forwarded to the underlying filesystem.
+
 
     Yields:
-        :obj:`tuple`: 3-tuple (dirpath, dirnames, filenames).
+        `tuple`: 3-tuple (dirpath, dirnames, filenames).
     """
-    main_hop, *rest_hops = str(urlpath).split("::")
+    main_hop, *rest_hops = _as_str(urlpath).split("::")
     if is_local_path(main_hop):
-        yield from os.walk(main_hop)
+        yield from os.walk(main_hop, **kwargs)
     else:
         # walking inside a zip in a private repo requires authentication
         if not rest_hops and (main_hop.startswith("http://") or main_hop.startswith("https://")):
@@ -558,22 +570,36 @@ def xwalk(urlpath, use_auth_token: Optional[Union[str, bool]] = None):
         inner_path = main_hop.split("://")[1]
         if inner_path.strip("/") and not fs.isdir(inner_path):
             return []
-        for dirpath, dirnames, filenames in fs.walk(inner_path):
-            yield "::".join([f"{fs.protocol}://{dirpath}"] + rest_hops), dirnames, filenames
+        protocol = fs.protocol if isinstance(fs.protocol, str) else fs.protocol[-1]
+        for dirpath, dirnames, filenames in fs.walk(inner_path, **kwargs):
+            yield "::".join([f"{protocol}://{dirpath}"] + rest_hops), dirnames, filenames
 
 
 class xPath(type(Path())):
+    """Extension of `pathlib.Path` to support both local paths and remote URLs."""
+
+    def __str__(self):
+        path_str = super().__str__()
+        main_hop, *rest_hops = path_str.split("::")
+        if is_local_path(main_hop):
+            return main_hop
+        path_as_posix = path_str.replace("\\", "/")
+        path_as_posix = SINGLE_SLASH_AFTER_PROTOCOL_PATTERN.sub("://", path_as_posix)
+        path_as_posix += "//" if path_as_posix.endswith(":") else ""  # Add slashes to root of the protocol
+        return path_as_posix
+
     def glob(self, pattern, use_auth_token: Optional[Union[str, bool]] = None):
         """Glob function for argument of type :obj:`~pathlib.Path` that supports both local paths end remote URLs.
 
         Args:
-            path (:obj:`~pathlib.Path`): Calling Path instance.
-            pattern (:obj:`str`): Pattern that resulting paths must match.
+            pattern (`str`): Pattern that resulting paths must match.
+            use_auth_token (`bool` or `str`, *optional*): Whether to use token or token to authenticate on the
+                Hugging Face Hub for private remote files.
 
         Yields:
-            :obj:`~pathlib.Path`
+            [`xPath`]
         """
-        posix_path = _as_posix(self)
+        posix_path = self.as_posix()
         main_hop, *rest_hops = posix_path.split("::")
         if is_local_path(main_hop):
             yield from Path(main_hop).glob(pattern)
@@ -599,11 +625,10 @@ class xPath(type(Path())):
         """Rglob function for argument of type :obj:`~pathlib.Path` that supports both local paths end remote URLs.
 
         Args:
-            path (:obj:`~pathlib.Path`): Calling Path instance.
-            pattern (:obj:`str`): Pattern that resulting paths must match.
+            pattern (`str`): Pattern that resulting paths must match.
 
         Yields:
-            :obj:`~pathlib.Path`
+            [`xPath`]
         """
         return self.glob("**/" + pattern, **kwargs)
 
@@ -611,77 +636,84 @@ class xPath(type(Path())):
     def parent(self) -> "xPath":
         """Name function for argument of type :obj:`~pathlib.Path` that supports both local paths end remote URLs.
 
-        Args:
-            path (:obj:`~pathlib.Path`): Calling Path instance.
-
         Returns:
-            :obj:`~pathlib.Path`
+            [`xPath`]
         """
-        return type(self)(xdirname(_as_posix(self)))
+        return type(self)(xdirname(self.as_posix()))
 
     @property
-    def name(self) -> PurePosixPath:
+    def name(self) -> str:
         """Name function for argument of type :obj:`~pathlib.Path` that supports both local paths end remote URLs.
 
-        Args:
-            path (:obj:`~pathlib.Path`): Calling Path instance.
-
         Returns:
-            :obj:`str`
+            `str`
         """
-        return PurePosixPath(_as_posix(self).split("::")[0]).name
+        return PurePosixPath(self.as_posix().split("::")[0]).name
 
     @property
-    def stem(self) -> PurePosixPath:
+    def stem(self) -> str:
         """Stem function for argument of type :obj:`~pathlib.Path` that supports both local paths end remote URLs.
 
-        Args:
-            path (:obj:`~pathlib.Path`): Calling Path instance.
-
         Returns:
-            :obj:`str`
+            `str`
         """
-        return PurePosixPath(_as_posix(self).split("::")[0]).stem
+        return PurePosixPath(self.as_posix().split("::")[0]).stem
 
     @property
-    def suffix(self) -> PurePosixPath:
+    def suffix(self) -> str:
         """Suffix function for argument of type :obj:`~pathlib.Path` that supports both local paths end remote URLs.
 
-        Args:
-            path (:obj:`~pathlib.Path`): Calling Path instance.
-
         Returns:
-            :obj:`str`
+            `str`
         """
-        return PurePosixPath(_as_posix(self).split("::")[0]).suffix
+        return PurePosixPath(self.as_posix().split("::")[0]).suffix
 
     def open(self, *args, **kwargs):
         """Extend :func:`xopen` to support argument of type :obj:`~pathlib.Path`.
 
         Args:
-            path (:obj:`~pathlib.Path`): Calling Path instance.
+            **args: Arguments passed to :func:`fsspec.open`.
             **kwargs: Keyword arguments passed to :func:`fsspec.open`.
 
         Returns:
-            :obj:`io.FileIO`: File-like object.
+            `io.FileIO`: File-like object.
         """
-        return xopen(_as_posix(self), *args, **kwargs)
+        return xopen(str(self), *args, **kwargs)
 
     def joinpath(self, *p: Tuple[str, ...]) -> "xPath":
         """Extend :func:`xjoin` to support argument of type :obj:`~pathlib.Path`.
 
         Args:
-            a (:obj:`~pathlib.Path`): Calling Path instance.
-            *p (:obj:`tuple` of :obj:`str`): Other path components.
+            *p (`tuple` of `str`): Other path components.
 
         Returns:
-            obj:`str`
+            [`xPath`]
         """
-        return type(self)(xjoin(_as_posix(self), *p))
+        return type(self)(xjoin(self.as_posix(), *p))
 
     def __truediv__(self, p: str) -> "xPath":
         return self.joinpath(p)
 
+    def with_suffix(self, suffix):
+        main_hop, *rest_hops = str(self).split("::")
+        if is_local_path(main_hop):
+            return type(self)(str(super().with_suffix(suffix)))
+        return type(self)("::".join([type(self)(PurePosixPath(main_hop).with_suffix(suffix)).as_posix()] + rest_hops))
+
+
+def _as_str(path: Union[str, Path, xPath]):
+    return str(path) if isinstance(path, xPath) else str(xPath(str(path)))
+
+
+def xgzip_open(filepath_or_buffer, *args, use_auth_token: Optional[Union[str, bool]] = None, **kwargs):
+    import gzip
+
+    if hasattr(filepath_or_buffer, "read"):
+        return gzip.open(filepath_or_buffer, *args, **kwargs)
+    else:
+        filepath_or_buffer = str(filepath_or_buffer)
+        return gzip.open(xopen(filepath_or_buffer, "rb", use_auth_token=use_auth_token), *args, **kwargs)
+
 
 def xpandas_read_csv(filepath_or_buffer, use_auth_token: Optional[Union[str, bool]] = None, **kwargs):
     import pandas as pd
@@ -715,12 +747,12 @@ def xet_parse(source, parser=None, use_auth_token: Optional[Union[str, bool]] =
 
     Args:
         source: File path or file object.
-        parser (optional, default `XMLParser`): Parser instance.
-        use_auth_token (:obj:`bool` or :obj:`str`, optional): Whether to use token or token to authenticate on the
+        parser (`XMLParser`, *optional*, default `XMLParser`): Parser instance.
+        use_auth_token (`bool` or `str`, optional): Whether to use token or token to authenticate on the
             Hugging Face Hub for private remote files.
 
     Returns:
-        :obj:`xml.etree.ElementTree.Element`: Root element of the given source document.
+        `xml.etree.ElementTree.Element`: Root element of the given source document.
     """
     if hasattr(source, "read"):
         return ET.parse(source, parser=parser)
@@ -816,11 +848,11 @@ class FilesIterable(_IterableFromGenerator):
                     # skipping hidden directories; prune the search
                     # [:] for the in-place list modification required by os.walk
                     # (only works for local paths as fsspec's walk doesn't support the in-place modification)
-                    dirnames[:] = [dirname for dirname in dirnames if not dirname.startswith((".", "__"))]
+                    dirnames[:] = sorted([dirname for dirname in dirnames if not dirname.startswith((".", "__"))])
                     if xbasename(dirpath).startswith((".", "__")):
                         # skipping hidden directories
                         continue
-                    for filename in filenames:
+                    for filename in sorted(filenames):
                         if filename.startswith((".", "__")):
                             # skipping hidden files
                             continue
@@ -834,9 +866,9 @@ class FilesIterable(_IterableFromGenerator):
 class StreamingDownloadManager:
     """
     Download manager that uses the "::" separator to navigate through (possibly remote) compressed archives.
-    Contrary to the regular DownloadManager, the `download` and `extract` methods don't actually download nor extract
+    Contrary to the regular `DownloadManager`, the `download` and `extract` methods don't actually download nor extract
     data, but they rather return the path or url that could be opened using the `xopen` function which extends the
-    builtin `open` function to stream data from remote files.
+    built-in `open` function to stream data from remote files.
     """
 
     is_streaming = True
@@ -858,15 +890,15 @@ class StreamingDownloadManager:
         return self._data_dir
 
     def download(self, url_or_urls):
-        """Download given url(s).
+        """Normalize URL(s) of files to stream data from.
+        This is the lazy version of `DownloadManager.download` for streaming.
 
         Args:
-            url_or_urls: url or `list`/`dict` of urls to download and extract. Each
-                url is a `str`.
+            url_or_urls (`str` or `list` or `dict`):
+                URL(s) of files to stream data from. Each url is a `str`.
 
         Returns:
-            downloaded_path(s): `str`, The downloaded paths matching the given input
-                url_or_urls.
+            url(s): (`str` or `list` or `dict`), URL(s) to stream data from matching the given input url_or_urls.
 
         Example:
 
@@ -884,18 +916,17 @@ class StreamingDownloadManager:
             urlpath = url_or_path_join(self._base_path, urlpath)
         return urlpath
 
-    def extract(self, path_or_paths):
-        """Extract given path(s).
+    def extract(self, url_or_urls):
+        """Add extraction protocol for given url(s) for streaming.
+
+        This is the lazy version of `DownloadManager.extract` for streaming.
 
         Args:
-            path_or_paths: path or `list`/`dict` of path of file to extract. Each
-                path is a `str`.
-            num_proc: Use multi-processing if `num_proc` > 1 and the length of
-                `path_or_paths` is larger than `num_proc`
+            url_or_urls (`str` or `list` or `dict`):
+                URL(s) of files to stream data from. Each url is a `str`.
 
         Returns:
-            extracted_path(s): `str`, The extracted paths matching the given input
-                path_or_paths.
+            url(s): (`str` or `list` or `dict`), URL(s) to stream data from matching the given input `url_or_urls`.
 
         Example:
 
@@ -904,7 +935,7 @@ class StreamingDownloadManager:
         >>> extracted_files = dl_manager.extract(downloaded_files)
         ```
         """
-        urlpaths = map_nested(self._extract, path_or_paths, map_tuple=True)
+        urlpaths = map_nested(self._extract, url_or_urls, map_tuple=True)
         return urlpaths
 
     def _extract(self, urlpath: str) -> str:
@@ -917,29 +948,27 @@ class StreamingDownloadManager:
             # there is one single file which is the uncompressed file
             inner_file = os.path.basename(urlpath.split("::")[0])
             inner_file = inner_file[: inner_file.rindex(".")] if "." in inner_file else inner_file
-            # check for tar.gz, tar.bz2 etc.
-            if inner_file.endswith(".tar"):
-                return f"tar://::{protocol}://{inner_file}::{urlpath}"
-            else:
-                return f"{protocol}://{inner_file}::{urlpath}"
+            return f"{protocol}://{inner_file}::{urlpath}"
         else:
             return f"{protocol}://::{urlpath}"
 
     def download_and_extract(self, url_or_urls):
-        """Download and extract given url_or_urls.
+        """Prepare given `url_or_urls` for streaming (add extraction protocol).
 
-        Is roughly equivalent to:
+        This is the lazy version of `DownloadManager.download_and_extract` for streaming.
+
+        Is equivalent to:
 
         ```
-        extracted_paths = dl_manager.extract(dl_manager.download(url_or_urls))
+        urls = dl_manager.extract(dl_manager.download(url_or_urls))
         ```
 
         Args:
-            url_or_urls: url or `list`/`dict` of urls to download and extract. Each
-                url is a `str`.
+            url_or_urls (`str` or `list` or `dict`):
+                URL(s) to stream from data from. Each url is a `str`.
 
         Returns:
-            extracted_path(s): `str`, extracted paths of given URL(s).
+            url(s): (`str` or `list` or `dict`), URL(s) to stream data from matching the given input `url_or_urls`.
         """
         return self.extract(self.download(url_or_urls))
 
@@ -947,10 +976,12 @@ class StreamingDownloadManager:
         """Iterate over files within an archive.
 
         Args:
-            urlpath_or_buf (:obj:`str` or :obj:`io.BufferedReader`): Archive path or archive binary file object.
+            urlpath_or_buf (`str` or `io.BufferedReader`):
+                Archive path or archive binary file object.
 
         Yields:
-            :obj:`tuple`[:obj:`str`, :obj:`io.BufferedReader`]: 2-tuple (path_within_archive, file_object).
+            `tuple[str, io.BufferedReader]`:
+                2-tuple (path_within_archive, file_object).
                 File object is opened in binary mode.
 
         Example:
@@ -970,7 +1001,8 @@ class StreamingDownloadManager:
         """Iterate over files.
 
         Args:
-            urlpaths (:obj:`str` or :obj:`list` of :obj:`str`): Root paths.
+            urlpaths (`str` or `list` of `str`):
+                Root paths.
 
         Yields:
             str: File URL path.
