@@ -1,4 +1,5 @@
 import os
+import warnings
 from dataclasses import dataclass, field
 from io import BytesIO
 from typing import TYPE_CHECKING, Any, ClassVar, Dict, Optional, Union
@@ -17,33 +18,39 @@ if TYPE_CHECKING:
     from .features import FeatureType
 
 
+_ffmpeg_warned, _librosa_warned, _audioread_warned = False, False, False
+
+
 @dataclass
 class Audio:
-    """Audio Feature to extract audio data from an audio file.
+    """Audio [`Feature`] to extract audio data from an audio file.
 
     Input: The Audio feature accepts as input:
-    - A :obj:`str`: Absolute path to the audio file (i.e. random access is allowed).
-    - A :obj:`dict` with the keys:
+    - A `str`: Absolute path to the audio file (i.e. random access is allowed).
+    - A `dict` with the keys:
 
-        - path: String with relative path of the audio file to the archive file.
-        - bytes: Bytes content of the audio file.
+        - `path`: String with relative path of the audio file to the archive file.
+        - `bytes`: Bytes content of the audio file.
 
       This is useful for archived files with sequential access.
 
-    - A :obj:`dict` with the keys:
+    - A `dict` with the keys:
 
-        - path: String with relative path of the audio file to the archive file.
-        - array: Array containing the audio sample
-        - sampling_rate: Integer corresponding to the samping rate of the audio sample.
+        - `path`: String with relative path of the audio file to the archive file.
+        - `array`: Array containing the audio sample
+        - `sampling_rate`: Integer corresponding to the sampling rate of the audio sample.
 
       This is useful for archived files with sequential access.
 
     Args:
-        sampling_rate (:obj:`int`, optional): Target sampling rate. If `None`, the native sampling rate is used.
-        mono (:obj:`bool`, default ``True``): Whether to convert the audio signal to mono by averaging samples across
+        sampling_rate (`int`, *optional*):
+            Target sampling rate. If `None`, the native sampling rate is used.
+        mono (`bool`, defaults to `True`):
+            Whether to convert the audio signal to mono by averaging samples across
             channels.
-        decode (:obj:`bool`, default ``True``): Whether to decode the audio data. If `False`,
-            returns the underlying dictionary in the format {"path": audio_path, "bytes": audio_bytes}.
+        decode (`bool`, defaults to `True`):
+            Whether to decode the audio data. If `False`,
+            returns the underlying dictionary in the format `{"path": audio_path, "bytes": audio_bytes}`.
 
     Example:
 
@@ -75,10 +82,11 @@ class Audio:
         """Encode example into a format for Arrow.
 
         Args:
-            value (:obj:`str` or :obj:`dict`): Data passed as input to Audio feature.
+            value (`str` or `dict`):
+                Data passed as input to Audio feature.
 
         Returns:
-            :obj:`dict`
+            `dict`
         """
         try:
             import soundfile as sf  # soundfile is a dependency of librosa, needed to decode audio files.
@@ -123,16 +131,18 @@ class Audio:
         """Decode example audio file into audio data.
 
         Args:
-            value (:obj:`dict`): a dictionary with keys:
+            value (`dict`):
+                A dictionary with keys:
 
-                - path: String with relative audio file path.
-                - bytes: Bytes of the audio file.
-            token_per_repo_id (:obj:`dict`, optional): To access and decode
+                - `path`: String with relative audio file path.
+                - `bytes`: Bytes of the audio file.
+            token_per_repo_id (`dict`, *optional*):
+                To access and decode
                 audio files from private repositories on the Hub, you can pass
-                a dictionary repo_id (str) -> token (bool or str)
+                a dictionary repo_id (`str`) -> token (`bool` or `str`)
 
         Returns:
-            dict
+            `dict`
         """
         if not self.decode:
             raise RuntimeError("Decoding is disabled for this feature. Please use Audio(decode=True) instead.")
@@ -171,17 +181,18 @@ class Audio:
         """Cast an Arrow array to the Audio arrow storage type.
         The Arrow types that can be converted to the Audio pyarrow storage type are:
 
-        - pa.string() - it must contain the "path" data
-        - pa.struct({"bytes": pa.binary()})
-        - pa.struct({"path": pa.string()})
-        - pa.struct({"bytes": pa.binary(), "path": pa.string()})  - order doesn't matter
+        - `pa.string()` - it must contain the "path" data
+        - `pa.struct({"bytes": pa.binary()})`
+        - `pa.struct({"path": pa.string()})`
+        - `pa.struct({"bytes": pa.binary(), "path": pa.string()})`  - order doesn't matter
 
         Args:
-            storage (Union[pa.StringArray, pa.StructArray]): PyArrow array to cast.
+            storage (`Union[pa.StringArray, pa.StructArray]`):
+                PyArrow array to cast.
 
         Returns:
-            pa.StructArray: Array in the Audio arrow storage type, that is
-                pa.struct({"bytes": pa.binary(), "path": pa.string()})
+            `pa.StructArray`: Array in the Audio arrow storage type, that is
+                `pa.struct({"bytes": pa.binary(), "path": pa.string()})`
         """
         if pa.types.is_string(storage.type):
             bytes_array = pa.array([None] * len(storage), type=pa.binary())
@@ -200,16 +211,16 @@ class Audio:
             storage = pa.StructArray.from_arrays([bytes_array, path_array], ["bytes", "path"], mask=storage.is_null())
         return array_cast(storage, self.pa_type)
 
-    def embed_storage(self, storage: pa.StructArray, drop_paths: bool = True) -> pa.StructArray:
+    def embed_storage(self, storage: pa.StructArray) -> pa.StructArray:
         """Embed audio files into the Arrow array.
 
         Args:
-            storage (pa.StructArray): PyArrow array to embed.
-            drop_paths (bool, default ``True``): If True, the paths are set to None.
+            storage (`pa.StructArray`):
+                PyArrow array to embed.
 
         Returns:
-            pa.StructArray: Array in the Audio arrow storage type, that is
-                pa.struct({"bytes": pa.binary(), "path": pa.string()})
+            `pa.StructArray`: Array in the Audio arrow storage type, that is
+                `pa.struct({"bytes": pa.binary(), "path": pa.string()})`.
         """
 
         @no_op_if_value_is_null
@@ -225,7 +236,10 @@ class Audio:
             ],
             type=pa.binary(),
         )
-        path_array = pa.array([None] * len(storage), type=pa.string()) if drop_paths else storage.field("path")
+        path_array = pa.array(
+            [os.path.basename(path) if path is not None else None for path in storage.field("path").to_pylist()],
+            type=pa.string(),
+        )
         storage = pa.StructArray.from_arrays([bytes_array, path_array], ["bytes", "path"], mask=bytes_array.is_null())
         return array_cast(storage, self.pa_type)
 
@@ -268,27 +282,72 @@ class Audio:
             if version.parse(sf.__libsndfile_version__) < version.parse("1.0.30"):
                 raise RuntimeError(
                     "Decoding .opus files requires 'libsndfile'>=1.0.30, "
-                    + "it can be installed via conda: `conda install -c conda-forge libsndfile>=1.0.30`"
+                    + 'it can be installed via conda: `conda install -c conda-forge "libsndfile>=1.0.30"`'
                 )
         array, sampling_rate = sf.read(file)
         array = array.T
         if self.mono:
             array = librosa.to_mono(array)
         if self.sampling_rate and self.sampling_rate != sampling_rate:
-            array = librosa.resample(array, sampling_rate, self.sampling_rate, res_type="kaiser_best")
+            array = librosa.resample(
+                array, orig_sr=sampling_rate, target_sr=self.sampling_rate, res_type="kaiser_best"
+            )
             sampling_rate = self.sampling_rate
         return array, sampling_rate
 
     def _decode_mp3(self, path_or_file):
         try:
             import torchaudio
-            import torchaudio.transforms as T
         except ImportError as err:
             raise ImportError("To support decoding 'mp3' audio files, please install 'torchaudio'.") from err
-        try:
-            torchaudio.set_audio_backend("sox_io")
-        except RuntimeError as err:
-            raise ImportError("To support decoding 'mp3' audio files, please install 'sox'.") from err
+        if version.parse(torchaudio.__version__) < version.parse("0.12.0"):
+            try:
+                torchaudio.set_audio_backend("sox_io")
+            except RuntimeError as err:
+                raise ImportError("To support decoding 'mp3' audio files, please install 'sox'.") from err
+            array, sampling_rate = self._decode_mp3_torchaudio(path_or_file)
+        else:
+            try:  # try torchaudio anyway because sometimes it works (depending on the os and os packages installed)
+                array, sampling_rate = self._decode_mp3_torchaudio(path_or_file)
+            except RuntimeError:
+                global _ffmpeg_warned
+                if not _ffmpeg_warned:
+                    warnings.warn(
+                        "\nTo support 'mp3' decoding with `torchaudio>=0.12.0`, please install `ffmpeg4` system package. On Google Colab you can run:\n\n"
+                        "\t!add-apt-repository -y ppa:jonathonf/ffmpeg-4 && apt update && apt install -y ffmpeg\n\n"
+                        "and restart your runtime. Alternatively, you can downgrade `torchaudio`:\n\n"
+                        "\tpip install \"torchaudio<0.12\"`.\n\nOtherwise 'mp3' files will be decoded with `librosa`."
+                    )
+                    _ffmpeg_warned = True
+                try:
+                    # flake8: noqa
+                    import librosa
+                except ImportError as err:
+                    raise ImportError(
+                        "To support 'mp3' decoding with `torchaudio>=0.12.0`, please install `ffmpeg4` system package. On Google Colab you can run:\n\n"
+                        "\t!add-apt-repository -y ppa:jonathonf/ffmpeg-4 && apt update && apt install -y ffmpeg\n\n"
+                        "and restart your runtime. Alternatively, you can downgrade `torchaudio`:\n\n"
+                        "\tpip install \"torchaudio<0.12\".\n\nTo decode 'mp3' files without `torchaudio`, please install `librosa`:\n\n"
+                        "\tpip install librosa\n\nNote that decoding might be extremely slow in that case."
+                    ) from err
+                # try to decode with librosa for torchaudio>=0.12.0 as a workaround
+                global _librosa_warned
+                if not _librosa_warned:
+                    warnings.warn("Decoding mp3 with `librosa` instead of `torchaudio`, decoding might be slow.")
+                    _librosa_warned = True
+                try:
+                    array, sampling_rate = self._decode_mp3_librosa(path_or_file)
+                except RuntimeError as err:
+                    raise RuntimeError(
+                        "Decoding of 'mp3' failed, probably because of streaming mode "
+                        "(`librosa` cannot decode 'mp3' file-like objects, only path-like)."
+                    ) from err
+
+        return array, sampling_rate
+
+    def _decode_mp3_torchaudio(self, path_or_file):
+        import torchaudio
+        import torchaudio.transforms as T
 
         array, sampling_rate = torchaudio.load(path_or_file, format="mp3")
         if self.sampling_rate and self.sampling_rate != sampling_rate:
@@ -300,3 +359,17 @@ class Audio:
         if self.mono:
             array = array.mean(axis=0)
         return array, sampling_rate
+
+    def _decode_mp3_librosa(self, path_or_file):
+        import librosa
+
+        global _audioread_warned
+
+        with warnings.catch_warnings():
+            if _audioread_warned:
+                warnings.filterwarnings("ignore", "pysoundfile failed.+?", UserWarning, module=librosa.__name__)
+            else:
+                _audioread_warned = True
+            array, sampling_rate = librosa.load(path_or_file, mono=self.mono, sr=self.sampling_rate)
+
+        return array, sampling_rate
