@@ -5,7 +5,6 @@ import random
 import shutil
 import tempfile
 import weakref
-from dataclasses import asdict
 from functools import wraps
 from pathlib import Path
 from typing import TYPE_CHECKING, Any, Dict, List, Optional, Union
@@ -15,10 +14,11 @@ import pyarrow as pa
 import xxhash
 
 from .info import DatasetInfo
+from .naming import INVALID_WINDOWS_CHARACTERS_IN_PATH
 from .table import ConcatenationTable, InMemoryTable, MemoryMappedTable, Table
 from .utils.deprecation_utils import deprecated
 from .utils.logging import get_logger
-from .utils.py_utils import dumps
+from .utils.py_utils import asdict, dumps
 
 
 if TYPE_CHECKING:
@@ -106,9 +106,9 @@ def enable_caching():
     - cache files are always recreated
     - cache files are written to a temporary directory that is deleted when session closes
     - cache files are named using a random hash instead of the dataset fingerprint
-    - use :func:`datasets.Dataset.save_to_disk` to save a transformed dataset or it will be deleted when session closes
-    - caching doesn't affect :func:`datasets.load_dataset`. If you want to regenerate a dataset from scratch you should use
-    the ``download_mode`` parameter in :func:`datasets.load_dataset`.
+    - use [`~datasets.Dataset.save_to_disk`] to save a transformed dataset or it will be deleted when session closes
+    - caching doesn't affect [`~datasets.load_dataset`]. If you want to regenerate a dataset from scratch you should use
+    the `download_mode` parameter in [`~datasets.load_dataset`].
     """
     global _CACHING_ENABLED
     _CACHING_ENABLED = True
@@ -127,9 +127,9 @@ def disable_caching():
     - cache files are always recreated
     - cache files are written to a temporary directory that is deleted when session closes
     - cache files are named using a random hash instead of the dataset fingerprint
-    - use :func:`datasets.Dataset.save_to_disk` to save a transformed dataset or it will be deleted when session closes
-    - caching doesn't affect :func:`datasets.load_dataset`. If you want to regenerate a dataset from scratch you should use
-    the ``download_mode`` parameter in :func:`datasets.load_dataset`.
+    - use [`~datasets.Dataset.save_to_disk`] to save a transformed dataset or it will be deleted when session closes
+    - caching doesn't affect [`~datasets.load_dataset`]. If you want to regenerate a dataset from scratch you should use
+    the `download_mode` parameter in [`~datasets.load_dataset`].
     """
     global _CACHING_ENABLED
     _CACHING_ENABLED = False
@@ -172,9 +172,9 @@ def is_caching_enabled() -> bool:
     - cache files are always recreated
     - cache files are written to a temporary directory that is deleted when session closes
     - cache files are named using a random hash instead of the dataset fingerprint
-    - use :func:`datasets.Dataset.save_to_disk` to save a transformed dataset or it will be deleted when session closes
-    - caching doesn't affect :func:`datasets.load_dataset`. If you want to regenerate a dataset from scratch you should use
-    the ``download_mode`` parameter in :func:`datasets.load_dataset`.
+    - use [`~datasets.Dataset.save_to_disk`]] to save a transformed dataset or it will be deleted when session closes
+    - caching doesn't affect [`~datasets.load_dataset`]. If you want to regenerate a dataset from scratch you should use
+    the `download_mode` parameter in [`~datasets.load_dataset`].
     """
     global _CACHING_ENABLED
     return bool(_CACHING_ENABLED)
@@ -344,6 +344,26 @@ def update_fingerprint(fingerprint, transform, transform_args):
     return hasher.hexdigest()
 
 
+def validate_fingerprint(fingerprint: str, max_length=64):
+    """
+    Make sure the fingerprint is a non-empty string that is not longer that max_length=64 by default,
+    so that the fingerprint can be used to name cache files without issues.
+    """
+    if not isinstance(fingerprint, str) or not fingerprint:
+        raise ValueError(f"Invalid fingerprint '{fingerprint}': it should be a non-empty string.")
+    for invalid_char in INVALID_WINDOWS_CHARACTERS_IN_PATH:
+        if invalid_char in fingerprint:
+            raise ValueError(
+                f"Invalid fingerprint. Bad characters from black list '{INVALID_WINDOWS_CHARACTERS_IN_PATH}' found in '{fingerprint}'. "
+                f"They could create issues when creating cache files."
+            )
+    if len(fingerprint) > max_length:
+        raise ValueError(
+            f"Invalid fingerprint. Maximum lenth is {max_length} but '{fingerprint}' has length {len(fingerprint)}."
+            "It could create issues when creating cache files."
+        )
+
+
 def fingerprint_transform(
     inplace: bool,
     use_kwargs: Optional[List[str]] = None,
@@ -452,6 +472,8 @@ def fingerprint_transform(
                         kwargs[fingerprint_name] = update_fingerprint(
                             self._fingerprint, transform, kwargs_for_fingerprint
                         )
+                    else:
+                        validate_fingerprint(kwargs[fingerprint_name])
 
             # Call actual function
 
