@@ -1,4 +1,3 @@
-# coding=utf-8
 # Copyright 2020 The HuggingFace Datasets Authors and the TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,30 +15,34 @@
 # flake8: noqa
 # Lint as: python3
 
-from typing import Dict, List, Optional
+from typing import Dict, List, Optional, Type
 
 from .. import config
-from ..utils.logging import get_logger
+from ..utils import logging
 from .formatting import (
     ArrowFormatter,
     CustomFormatter,
     Formatter,
-    NumpyFormatter,
     PandasFormatter,
     PythonFormatter,
     format_table,
     query_table,
 )
+from .np_formatter import NumpyFormatter
 
 
-logger = get_logger(__name__)
+logger = logging.get_logger(__name__)
 
-_FORMAT_TYPES: Dict[Optional[str], type] = {}
+_FORMAT_TYPES: Dict[Optional[str], Type[Formatter]] = {}
 _FORMAT_TYPES_ALIASES: Dict[Optional[str], str] = {}
 _FORMAT_TYPES_ALIASES_UNAVAILABLE: Dict[Optional[str], Exception] = {}
 
 
-def _register_formatter(formatter_cls: type, format_type: Optional[str], aliases: Optional[List[str]] = None):
+def _register_formatter(
+    formatter_cls: type,
+    format_type: Optional[str],
+    aliases: Optional[List[str]] = None,
+):
     """
     Register a Formatter object using a name and optional aliases.
     This function must be used on a Formatter class.
@@ -124,7 +127,5 @@ def get_formatter(format_type: Optional[str], **format_kwargs) -> Formatter:
         raise _FORMAT_TYPES_ALIASES_UNAVAILABLE[format_type]
     else:
         raise ValueError(
-            "Return type should be None or selected in {}, but got '{}'".format(
-                list(type for type in _FORMAT_TYPES.keys() if type != None), format_type
-            )
+            f"Return type should be None or selected in {list(type for type in _FORMAT_TYPES.keys() if type != None)}, but got '{format_type}'"
         )
