@@ -13,7 +13,7 @@
 # limitations under the License.
 
 # Lint as: python3
-""" List and inspect datasets and metrics."""
+""" List and inspect datasets."""
 
 import inspect
 import os
@@ -28,6 +28,7 @@ from .download.download_manager import DownloadMode
 from .download.streaming_download_manager import StreamingDownloadManager
 from .info import DatasetInfo
 from .load import dataset_module_factory, import_main_class, load_dataset_builder, metric_module_factory
+from .utils.deprecation_utils import deprecated
 from .utils.file_utils import relative_to_absolute_path
 from .utils.logging import get_logger
 from .utils.version import Version
@@ -44,8 +45,10 @@ def list_datasets(with_community_datasets=True, with_details=False):
     """List all the datasets scripts available on the Hugging Face Hub.
 
     Args:
-        with_community_datasets (:obj:`bool`, optional, default ``True``): Include the community provided datasets.
-        with_details (:obj:`bool`, optional, default ``False``): Return the full details on the datasets instead of only the short name.
+        with_community_datasets (`bool`, *optional*, defaults to `True`):
+            Include the community provided datasets.
+        with_details (`bool`, *optional*, defaults to `False`):
+            Return the full details on the datasets instead of only the short name.
 
     Example:
 
@@ -70,9 +73,18 @@ def list_datasets(with_community_datasets=True, with_details=False):
     return datasets
 
 
+@deprecated(
+    "Use 'evaluate.list_evaluation_modules' instead, from the new library 🤗 Evaluate: https://huggingface.co/docs/evaluate"
+)
 def list_metrics(with_community_metrics=True, with_details=False):
     """List all the metrics script available on the Hugging Face Hub.
 
+    <Deprecated version="2.5.0">
+
+    Use `evaluate.list_evaluation_modules` instead, from the new library 🤗 Evaluate: https://huggingface.co/docs/evaluate
+
+    </Deprecated>
+
     Args:
         with_community_metrics (:obj:`bool`, optional, default ``True``): Include the community provided metrics.
         with_details (:obj:`bool`, optional, default ``False``): Return the full details on the metrics instead of only the short name.
@@ -109,12 +121,15 @@ def inspect_dataset(path: str, local_path: str, download_config: Optional[Downlo
 
             - a local path to processing script or the directory containing the script (if the script has the same name
                 as the directory),
-                e.g. ``'./dataset/squad'`` or ``'./dataset/squad/squad.py'``.
+                e.g. `'./dataset/squad'` or `'./dataset/squad/squad.py'`.
             - a dataset identifier on the Hugging Face Hub (list all available datasets and ids with [`list_datasets`])
-                e.g. ``'squad'``, ``'glue'`` or ``'openai/webtext'``.
-        local_path (`str`): Path to the local folder to copy the dataset script to.
-        download_config ([`DownloadConfig`], *optional*): Specific download configuration parameters.
-        **download_kwargs (additional keyword arguments): Optional arguments for [`DownloadConfig`] which will override
+                e.g. `'squad'`, `'glue'` or `'openai/webtext'`.
+        local_path (`str`):
+            Path to the local folder to copy the dataset script to.
+        download_config ([`DownloadConfig`], *optional*):
+            Specific download configuration parameters.
+        **download_kwargs (additional keyword arguments):
+            Optional arguments for [`DownloadConfig`] which will override
             the attributes of `download_config` if supplied.
     """
     dataset_module = dataset_module_factory(path, download_config=download_config, **download_kwargs)
@@ -138,10 +153,19 @@ def inspect_dataset(path: str, local_path: str, download_config: Optional[Downlo
     )
 
 
+@deprecated(
+    "Use 'evaluate.inspect_evaluation_module' instead, from the new library 🤗 Evaluate: https://huggingface.co/docs/evaluate"
+)
 def inspect_metric(path: str, local_path: str, download_config: Optional[DownloadConfig] = None, **download_kwargs):
     r"""
     Allow inspection/modification of a metric script by copying it on local drive at local_path.
 
+    <Deprecated version="2.5.0">
+
+    Use `evaluate.inspect_evaluation_module` instead, from the new library 🤗 Evaluate instead: https://huggingface.co/docs/evaluate
+
+    </Deprecated>
+
     Args:
         path (``str``): path to the dataset processing script with the dataset builder. Can be either:
 
@@ -185,24 +209,29 @@ def get_dataset_infos(
     """Get the meta information about a dataset, returned as a dict mapping config name to DatasetInfoDict.
 
     Args:
-        path (``str``): path to the dataset processing script with the dataset builder. Can be either:
+        path (`str`): path to the dataset processing script with the dataset builder. Can be either:
 
             - a local path to processing script or the directory containing the script (if the script has the same name as the directory),
-                e.g. ``'./dataset/squad'`` or ``'./dataset/squad/squad.py'``
-            - a dataset identifier on the Hugging Face Hub (list all available datasets and ids with ``datasets.list_datasets()``)
-                e.g. ``'squad'``, ``'glue'`` or ``'openai/webtext'``
-        revision (Optional ``Union[str, datasets.Version]``):
+                e.g. `'./dataset/squad'` or `'./dataset/squad/squad.py'`
+            - a dataset identifier on the Hugging Face Hub (list all available datasets and ids with [`datasets.list_datasets`])
+                e.g. `'squad'`, `'glue'` or``'openai/webtext'`
+        revision (`Union[str, datasets.Version]`, *optional*):
             If specified, the dataset module will be loaded from the datasets repository at this version.
             By default:
             - it is set to the local version of the lib.
             - it will also try to load it from the main branch if it's not available at the local version of the lib.
             Specifying a version that is different from your local version of the lib might cause compatibility issues.
-        download_config (:class:`DownloadConfig`, optional): Specific download configuration parameters.
-        download_mode (:class:`DownloadMode`, default ``REUSE_DATASET_IF_EXISTS``): Download/generate mode.
-        data_files (:obj:`Union[Dict, List, str]`, optional): Defining the data_files of the dataset configuration.
-        use_auth_token (``str`` or :obj:`bool`, optional): Optional string or boolean to use as Bearer token for remote files on the Datasets Hub.
-            If True, will get token from `"~/.huggingface"`.
-        **config_kwargs (additional keyword arguments): optional attributes for builder class which will override the attributes if supplied.
+        download_config ([`DownloadConfig`], *optional*):
+            Specific download configuration parameters.
+        download_mode ([`DownloadMode`], defaults to `REUSE_DATASET_IF_EXISTS`):
+            Download/generate mode.
+        data_files (`Union[Dict, List, str]`, *optional*):
+            Defining the data_files of the dataset configuration.
+        use_auth_token (`str` or `bool`, *optional*):
+            Optional string or boolean to use as Bearer token for remote files on the Datasets Hub.
+            If `True`, or not specified, will get token from `"~/.huggingface"`.
+        **config_kwargs (additional keyword arguments):
+            Optional attributes for builder class which will override the attributes if supplied.
 
     Example:
 
@@ -218,6 +247,7 @@ def get_dataset_infos(
         download_config=download_config,
         download_mode=download_mode,
         data_files=data_files,
+        use_auth_token=use_auth_token,
     )
     return {
         config_name: get_dataset_config_info(
@@ -246,26 +276,30 @@ def get_dataset_config_names(
     """Get the list of available config names for a particular dataset.
 
     Args:
-        path (``str``): path to the dataset processing script with the dataset builder. Can be either:
+        path (`str`): path to the dataset processing script with the dataset builder. Can be either:
 
             - a local path to processing script or the directory containing the script (if the script has the same name as the directory),
-                e.g. ``'./dataset/squad'`` or ``'./dataset/squad/squad.py'``
-            - a dataset identifier on the Hugging Face Hub (list all available datasets and ids with ``datasets.list_datasets()``)
-                e.g. ``'squad'``, ``'glue'`` or ``'openai/webtext'``
-        revision (Optional ``Union[str, datasets.Version]``):
+                e.g. `'./dataset/squad'` or `'./dataset/squad/squad.py'`
+            - a dataset identifier on the Hugging Face Hub (list all available datasets and ids with [`datasets.list_datasets`])
+                e.g. `'squad'`, `'glue'` or `'openai/webtext'`
+        revision (`Union[str, datasets.Version]`, *optional*):
             If specified, the dataset module will be loaded from the datasets repository at this version.
             By default:
             - it is set to the local version of the lib.
             - it will also try to load it from the main branch if it's not available at the local version of the lib.
             Specifying a version that is different from your local version of the lib might cause compatibility issues.
-        download_config (:class:`DownloadConfig`, optional): Specific download configuration parameters.
-        download_mode (:class:`DownloadMode`, default ``REUSE_DATASET_IF_EXISTS``): Download/generate mode.
-        dynamic_modules_path (Optional str, defaults to HF_MODULES_CACHE / "datasets_modules", i.e. ~/.cache/huggingface/modules/datasets_modules):
-            Optional path to the directory in which the dynamic modules are saved. It must have been initialized with :obj:`init_dynamic_modules`.
+        download_config ([`DownloadConfig`], *optional*):
+            Specific download configuration parameters.
+        download_mode ([`DownloadMode`], defaults to `REUSE_DATASET_IF_EXISTS`):
+            Download/generate mode.
+        dynamic_modules_path (`str`, defaults to `~/.cache/huggingface/modules/datasets_modules`):
+            Optional path to the directory in which the dynamic modules are saved. It must have been initialized with `init_dynamic_modules`.
             By default the datasets and metrics are stored inside the `datasets_modules` module.
-        data_files (:obj:`Union[Dict, List, str]`, optional): Defining the data_files of the dataset configuration.
-        **download_kwargs (additional keyword arguments): optional attributes for DownloadConfig() which will override the attributes in download_config if supplied,
-            for example ``use_auth_token``
+        data_files (`Union[Dict, List, str]`, *optional*):
+            Defining the data_files of the dataset configuration.
+        **download_kwargs (additional keyword arguments):
+            Optional attributes for [`DownloadConfig`] which will override the attributes in `download_config` if supplied,
+            for example `use_auth_token`.
 
     Example:
 
@@ -322,14 +356,11 @@ def get_dataset_config_info(
         data_files (:obj:`str` or :obj:`Sequence` or :obj:`Mapping`, optional): Path(s) to source data file(s).
         download_config (:class:`~download.DownloadConfig`, optional): Specific download configuration parameters.
         download_mode (:class:`DownloadMode`, default ``REUSE_DATASET_IF_EXISTS``): Download/generate mode.
-        revision (:class:`~utils.Version` or :obj:`str`, optional): Version of the dataset script to load:
-
-            - For datasets in the `huggingface/datasets` library on GitHub like "squad", the default version of the module is the local version of the lib.
-              You can specify a different version from your local version of the lib (e.g. "main" or "1.2.0") but it might cause compatibility issues.
-            - For community datasets like "lhoestq/squad" that have their own git repository on the Datasets Hub, the default version "main" corresponds to the "main" branch.
-              You can specify a different version that the default "main" by using a commit sha or a git tag of the dataset repository.
+        revision (:class:`~utils.Version` or :obj:`str`, optional): Version of the dataset script to load.
+            As datasets have their own git repository on the Datasets Hub, the default version "main" corresponds to their "main" branch.
+            You can specify a different version than the default "main" by using a commit SHA or a git tag of the dataset repository.
         use_auth_token (``str`` or :obj:`bool`, optional): Optional string or boolean to use as Bearer token for remote files on the Datasets Hub.
-            If True, will get token from `"~/.huggingface"`.
+            If True, or not specified, will get token from `"~/.huggingface"`.
         **config_kwargs (additional keyword arguments): optional attributes for builder class which will override the attributes if supplied.
 
     """
@@ -345,10 +376,13 @@ def get_dataset_config_info(
     )
     info = builder.info
     if info.splits is None:
+        download_config = download_config.copy() if download_config else DownloadConfig()
+        if use_auth_token is not None:
+            download_config.use_auth_token = use_auth_token
+        builder._check_manual_download(
+            StreamingDownloadManager(base_path=builder.base_path, download_config=download_config)
+        )
         try:
-            download_config = download_config.copy() if download_config else DownloadConfig()
-            if use_auth_token is not None:
-                download_config.use_auth_token = use_auth_token
             info.splits = {
                 split_generator.name: {"name": split_generator.name, "dataset_name": path}
                 for split_generator in builder._split_generators(
@@ -373,25 +407,29 @@ def get_dataset_split_names(
     """Get the list of available splits for a particular config and dataset.
 
     Args:
-        path (``str``): path to the dataset processing script with the dataset builder. Can be either:
+        path (`str`): path to the dataset processing script with the dataset builder. Can be either:
 
             - a local path to processing script or the directory containing the script (if the script has the same name as the directory),
-                e.g. ``'./dataset/squad'`` or ``'./dataset/squad/squad.py'``
-            - a dataset identifier on the Hugging Face Hub (list all available datasets and ids with ``datasets.list_datasets()``)
-                e.g. ``'squad'``, ``'glue'`` or ``'openai/webtext'``
-        config_name (:obj:`str`, optional): Defining the name of the dataset configuration.
-        data_files (:obj:`str` or :obj:`Sequence` or :obj:`Mapping`, optional): Path(s) to source data file(s).
-        download_config (:class:`~download.DownloadConfig`, optional): Specific download configuration parameters.
-        download_mode (:class:`DownloadMode`, default ``REUSE_DATASET_IF_EXISTS``): Download/generate mode.
-        revision (:class:`~utils.Version` or :obj:`str`, optional): Version of the dataset script to load:
-
-            - For datasets in the `huggingface/datasets` library on GitHub like "squad", the default version of the module is the local version of the lib.
-              You can specify a different version from your local version of the lib (e.g. "main" or "1.2.0") but it might cause compatibility issues.
-            - For community datasets like "lhoestq/squad" that have their own git repository on the Datasets Hub, the default version "main" corresponds to the "main" branch.
-              You can specify a different version that the default "main" by using a commit sha or a git tag of the dataset repository.
-        use_auth_token (``str`` or :obj:`bool`, optional): Optional string or boolean to use as Bearer token for remote files on the Datasets Hub.
-            If True, will get token from `"~/.huggingface"`.
-        **config_kwargs (additional keyword arguments): optional attributes for builder class which will override the attributes if supplied.
+                e.g. `'./dataset/squad'` or `'./dataset/squad/squad.py'`
+            - a dataset identifier on the Hugging Face Hub (list all available datasets and ids with [`datasets.list_datasets`])
+                e.g. `'squad'`, `'glue'` or `'openai/webtext'`
+        config_name (`str`, *optional*):
+            Defining the name of the dataset configuration.
+        data_files (`str` or `Sequence` or `Mapping`, *optional*):
+            Path(s) to source data file(s).
+        download_config ([`DownloadConfig`], *optional*):
+            Specific download configuration parameters.
+        download_mode ([`DownloadMode`], defaults to `REUSE_DATASET_IF_EXISTS`):
+            Download/generate mode.
+        revision ([`Version`] or `str`, *optional*):
+            Version of the dataset script to load.
+            As datasets have their own git repository on the Datasets Hub, the default version "main" corresponds to their "main" branch.
+            You can specify a different version than the default "main" by using a commit SHA or a git tag of the dataset repository.
+        use_auth_token (`str` or `bool`, *optional*):
+            Optional string or boolean to use as Bearer token for remote files on the Datasets Hub.
+            If `True`, or not specified, will get token from `"~/.huggingface"`.
+        **config_kwargs (additional keyword arguments):
+            Optional attributes for builder class which will override the attributes if supplied.
 
     Example:
 
