@@ -20,14 +20,19 @@ class ParquetDatasetReader(AbstractDatasetReader):
         features: Optional[Features] = None,
         cache_dir: str = None,
         keep_in_memory: bool = False,
+        streaming: bool = False,
+        num_proc: Optional[int] = None,
         **kwargs,
     ):
-        if config.PYARROW_VERSION.major < 3:
-            raise ImportError(
-                "PyArrow >= 3.0.0 is required to used the ParquetDatasetReader: pip install --upgrade pyarrow"
-            )
         super().__init__(
-            path_or_paths, split=split, features=features, cache_dir=cache_dir, keep_in_memory=keep_in_memory, **kwargs
+            path_or_paths,
+            split=split,
+            features=features,
+            cache_dir=cache_dir,
+            keep_in_memory=keep_in_memory,
+            streaming=streaming,
+            num_proc=num_proc,
+            **kwargs,
         )
         path_or_paths = path_or_paths if isinstance(path_or_paths, dict) else {self.split: path_or_paths}
         hash = _PACKAGED_DATASETS_MODULES["parquet"][1]
@@ -40,25 +45,29 @@ class ParquetDatasetReader(AbstractDatasetReader):
         )
 
     def read(self):
-        download_config = None
-        download_mode = None
-        ignore_verifications = False
-        use_auth_token = None
-        base_path = None
+        # Build iterable dataset
+        if self.streaming:
+            dataset = self.builder.as_streaming_dataset(split=self.split)
+        # Build regular (map-style) dataset
+        else:
+            download_config = None
+            download_mode = None
+            ignore_verifications = False
+            use_auth_token = None
+            base_path = None
 
-        self.builder.download_and_prepare(
-            download_config=download_config,
-            download_mode=download_mode,
-            ignore_verifications=ignore_verifications,
-            # try_from_hf_gcs=try_from_hf_gcs,
-            base_path=base_path,
-            use_auth_token=use_auth_token,
-        )
-
-        # Build dataset for splits
-        dataset = self.builder.as_dataset(
-            split=self.split, ignore_verifications=ignore_verifications, in_memory=self.keep_in_memory
-        )
+            self.builder.download_and_prepare(
+                download_config=download_config,
+                download_mode=download_mode,
+                ignore_verifications=ignore_verifications,
+                # try_from_hf_gcs=try_from_hf_gcs,
+                base_path=base_path,
+                use_auth_token=use_auth_token,
+                num_proc=self.num_proc,
+            )
+            dataset = self.builder.as_dataset(
+                split=self.split, ignore_verifications=ignore_verifications, in_memory=self.keep_in_memory
+            )
         return dataset
 
 
@@ -70,10 +79,6 @@ class ParquetDatasetWriter:
         batch_size: Optional[int] = None,
         **parquet_writer_kwargs,
     ):
-        if config.PYARROW_VERSION.major < 3:
-            raise ImportError(
-                "PyArrow >= 3.0.0 is required to used the ParquetDatasetWriter: pip install --upgrade pyarrow"
-            )
         self.dataset = dataset
         self.path_or_buf = path_or_buf
         self.batch_size = batch_size
@@ -107,4 +112,5 @@ class ParquetDatasetWriter:
             )
             writer.write_table(batch)
             written += batch.nbytes
+        writer.close()
         return written
