@@ -424,17 +424,17 @@ class IndexableMixin:
             )
 
     def list_indexes(self) -> List[str]:
-        """List the colindex_nameumns/identifiers of all the attached indexes."""
+        """List the `colindex_nameumns`/identifiers of all the attached indexes."""
         return list(self._indexes)
 
     def get_index(self, index_name: str) -> BaseIndex:
-        """List the index_name/identifiers of all the attached indexes.
+        """List the `index_name`/identifiers of all the attached indexes.
 
         Args:
-            index_name (:obj:`str`): Index name.
+            index_name (`str`): Index name.
 
         Returns:
-            :class:`BaseIndex`
+            [`BaseIndex`]
         """
         self._check_index_is_initialized(index_name)
         return self._indexes[index_name]
@@ -458,17 +458,18 @@ class IndexableMixin:
         - For `string factory`: https://github.com/facebookresearch/faiss/wiki/The-index-factory
 
         Args:
-            column (:obj:`str`): The column of the vectors to add to the index.
-            index_name (Optional :obj:`str`): The index_name/identifier of the index. This is the index_name that is used to call `.get_nearest` or `.search`.
-                By defaul it corresponds to `column`.
-            device (Optional :obj:`Union[int, List[int]]`): If positive integer, this is the index of the GPU to use. If negative integer, use all GPUs.
+            column (`str`): The column of the vectors to add to the index.
+            index_name (Optional `str`): The index_name/identifier of the index. This is the index_name that is used to call `.get_nearest` or `.search`.
+                By default it corresponds to `column`.
+            device (Optional `Union[int, List[int]]`): If positive integer, this is the index of the GPU to use. If negative integer, use all GPUs.
                 If a list of positive integers is passed in, run only on those GPUs. By default it uses the CPU.
-            string_factory (Optional :obj:`str`): This is passed to the index factory of Faiss to create the index. Default index class is IndexFlatIP.
-            metric_type (Optional :obj:`int`): Type of metric. Ex: `faiss.METRIC_INNER_PRODUCT` or `faiss.METRIC_L2`.
-            custom_index (Optional :obj:`faiss.Index`): Custom Faiss index that you already have instantiated and configured for your needs.
-            batch_size (Optional :obj:`int`): Size of the batch to use while adding vectors to the FaissIndex. Default value is 1000.
-            train_size (Optional :obj:`int`): If the index needs a training step, specifies how many vectors will be used to train the index.
-            faiss_verbose (:obj:`bool`, defaults to False): Enable the verbosity of the Faiss index.
+            string_factory (Optional `str`): This is passed to the index factory of Faiss to create the index. Default index class is IndexFlatIP.
+            metric_type (Optional `int`): Type of metric. Ex: `faiss.METRIC_INNER_PRODUCT` or `faiss.METRIC_L2`.
+            custom_index (Optional `faiss.Index`): Custom Faiss index that you already have instantiated and configured for your needs.
+            batch_size (Optional `int`): Size of the batch to use while adding vectors to the FaissIndex. Default value is 1000.
+                <Added version="2.4.0"/>
+            train_size (Optional `int`): If the index needs a training step, specifies how many vectors will be used to train the index.
+            faiss_verbose (`bool`, defaults to False): Enable the verbosity of the Faiss index.
         """
         index_name = index_name if index_name is not None else column
         faiss_index = FaissIndex(
@@ -498,17 +499,18 @@ class IndexableMixin:
         - For `string factory`: https://github.com/facebookresearch/faiss/wiki/The-index-factory
 
         Args:
-            external_arrays (:obj:`np.array`): If you want to use arrays from outside the lib for the index, you can set `external_arrays`.
+            external_arrays (`np.array`): If you want to use arrays from outside the lib for the index, you can set `external_arrays`.
                 It will use `external_arrays` to create the Faiss index instead of the arrays in the given `column`.
-            index_name (:obj:`str`): The index_name/identifier of the index. This is the index_name that is used to call `.get_nearest` or `.search`.
-            device (Optional :obj:`Union[int, List[int]]`): If positive integer, this is the index of the GPU to use. If negative integer, use all GPUs.
+            index_name (`str`): The index_name/identifier of the index. This is the index_name that is used to call `.get_nearest` or `.search`.
+            device (Optional `Union[int, List[int]]`): If positive integer, this is the index of the GPU to use. If negative integer, use all GPUs.
                 If a list of positive integers is passed in, run only on those GPUs. By default it uses the CPU.
-            string_factory (Optional :obj:`str`): This is passed to the index factory of Faiss to create the index. Default index class is IndexFlatIP.
-            metric_type (Optional :obj:`int`): Type of metric. Ex: `faiss.METRIC_INNER_PRODUCT` or `faiss.METRIC_L2`.
-            custom_index (Optional :obj:`faiss.Index`): Custom Faiss index that you already have instantiated and configured for your needs.
-            batch_size (Optional :obj:`int`): Size of the batch to use while adding vectors to the FaissIndex. Default value is 1000.
-            train_size (Optional :obj:`int`): If the index needs a training step, specifies how many vectors will be used to train the index.
-            faiss_verbose (:obj:`bool`, defaults to False): Enable the verbosity of the Faiss index.
+            string_factory (Optional `str`): This is passed to the index factory of Faiss to create the index. Default index class is IndexFlatIP.
+            metric_type (Optional `int`): Type of metric. Ex: `faiss.METRIC_INNER_PRODUCT` or `faiss.METRIC_L2`.
+            custom_index (Optional `faiss.Index`): Custom Faiss index that you already have instantiated and configured for your needs.
+            batch_size (Optional `int`): Size of the batch to use while adding vectors to the FaissIndex. Default value is 1000.
+                <Added version="2.4.0"/>
+            train_size (Optional `int`): If the index needs a training step, specifies how many vectors will be used to train the index.
+            faiss_verbose (`bool`, defaults to False): Enable the verbosity of the Faiss index.
         """
         faiss_index = FaissIndex(
             device=device, string_factory=string_factory, metric_type=metric_type, custom_index=custom_index
@@ -522,8 +524,8 @@ class IndexableMixin:
         """Save a FaissIndex on disk.
 
         Args:
-            index_name (:obj:`str`): The index_name/identifier of the index. This is the index_name that is used to call `.get_nearest` or `.search`.
-            file (:obj:`str`): The path to the serialized faiss index on disk.
+            index_name (`str`): The index_name/identifier of the index. This is the index_name that is used to call `.get_nearest` or `.search`.
+            file (`str`): The path to the serialized faiss index on disk.
         """
         index = self.get_index(index_name)
         if not isinstance(index, FaissIndex):
@@ -543,10 +545,10 @@ class IndexableMixin:
         `.get_index(index_name).faiss_index` to make it fit your needs.
 
         Args:
-            index_name (:obj:`str`): The index_name/identifier of the index. This is the index_name that is used to
+            index_name (`str`): The index_name/identifier of the index. This is the index_name that is used to
                 call `.get_nearest` or `.search`.
-            file (:obj:`str`): The path to the serialized faiss index on disk.
-            device (Optional :obj:`Union[int, List[int]]`): If positive integer, this is the index of the GPU to use. If negative integer, use all GPUs.
+            file (`str`): The path to the serialized faiss index on disk.
+            device (Optional `Union[int, List[int]]`): If positive integer, this is the index of the GPU to use. If negative integer, use all GPUs.
                 If a list of positive integers is passed in, run only on those GPUs. By default it uses the CPU.
         """
         index = FaissIndex.load(file, device=device)
@@ -570,17 +572,17 @@ class IndexableMixin:
         """Add a text index using ElasticSearch for fast retrieval.
 
         Args:
-            column (:obj:`str`): The column of the documents to add to the index.
-            index_name (Optional :obj:`str`): The index_name/identifier of the index. This is the index name that is used to call `.get_nearest` or `.search`.
-                By defaul it corresponds to `column`.
-            host (Optional :obj:`str`, defaults to localhost):
+            column (`str`): The column of the documents to add to the index.
+            index_name (Optional `str`): The index_name/identifier of the index. This is the index name that is used to call `.get_nearest` or `.search`.
+                By default it corresponds to `column`.
+            host (Optional `str`, defaults to localhost):
                 host of where ElasticSearch is running
-            port (Optional :obj:`str`, defaults to 9200):
+            port (Optional `str`, defaults to 9200):
                 port of where ElasticSearch is running
-            es_client (Optional :obj:`elasticsearch.Elasticsearch`):
+            es_client (Optional `elasticsearch.Elasticsearch`):
                 The elasticsearch client used to create the index if host and port are None.
-            es_index_name (Optional :obj:`str`): The elasticsearch index name used to create the index.
-            es_index_config (Optional :obj:`dict`):
+            es_index_name (Optional `str`): The elasticsearch index name used to create the index.
+            es_index_config (Optional `dict`):
                 The configuration of the elasticsearch index.
                 Default config is:
 
@@ -621,18 +623,20 @@ class IndexableMixin:
         """Load an existing text index using ElasticSearch for fast retrieval.
 
         Args:
-            index_name (:obj:`str`): The index_name/identifier of the index. This is the index name that is used to call `.get_nearest` or `.search`.
-            es_index_name (:obj:`str`): The name of elasticsearch index to load.
-            host (Optional :obj:`str`, defaults to localhost):
-                host of where ElasticSearch is running
-            port (Optional :obj:`str`, defaults to 9200):
-                port of where ElasticSearch is running
-            es_client (Optional :obj:`elasticsearch.Elasticsearch`):
-                The elasticsearch client used to create the index if host and port are None.
-            es_index_config (Optional :obj:`dict`):
+            index_name (`str`):
+                The `index_name`/identifier of the index. This is the index name that is used to call `get_nearest` or `search`.
+            es_index_name (`str`):
+                The name of elasticsearch index to load.
+            host (`str`, *optional*, defaults to `localhost`):
+                Host of where ElasticSearch is running.
+            port (`str`, *optional*, defaults to `9200`):
+                Port of where ElasticSearch is running.
+            es_client (`elasticsearch.Elasticsearch`, *optional*):
+                The elasticsearch client used to create the index if host and port are `None`.
+            es_index_config (`dict`, *optional*):
                 The configuration of the elasticsearch index.
-                Default config is::
-
+                Default config is:
+                    ```
                     {
                         "settings": {
                             "number_of_shards": 1,
@@ -648,6 +652,7 @@ class IndexableMixin:
                             }
                         },
                     }
+                    ```
         """
         self._indexes[index_name] = ElasticSearchIndex(
             host=host, port=port, es_client=es_client, es_index_name=es_index_name, es_index_config=es_index_config
@@ -657,7 +662,8 @@ class IndexableMixin:
         """Drop the index with the specified column.
 
         Args:
-            index_name (:obj:`str`): The index_name/identifier of the index.
+            index_name (`str`):
+                The `index_name`/identifier of the index.
         """
         del self._indexes[index_name]
 
@@ -665,13 +671,16 @@ class IndexableMixin:
         """Find the nearest examples indices in the dataset to the query.
 
         Args:
-            index_name (:obj:`str`): The name/identifier of the index.
-            query (:obj:`Union[str, np.ndarray]`): The query as a string if `index_name` is a text index or as a numpy array if `index_name` is a vector index.
-            k (:obj:`int`): The number of examples to retrieve.
+            index_name (`str`):
+                The name/identifier of the index.
+            query (`Union[str, np.ndarray]`):
+                The query as a string if `index_name` is a text index or as a numpy array if `index_name` is a vector index.
+            k (`int`):
+                The number of examples to retrieve.
 
         Returns:
-            scores (:obj:`List[List[float]`): The retrieval scores of the retrieved examples.
-            indices (:obj:`List[List[int]]`): The indices of the retrieved examples.
+            - scores (`List[List[float]`): The retrieval scores of the retrieved examples.
+            - indices (`List[List[int]]`): The indices of the retrieved examples.
         """
         self._check_index_is_initialized(index_name)
         return self._indexes[index_name].search(query, k)
@@ -680,13 +689,16 @@ class IndexableMixin:
         """Find the nearest examples indices in the dataset to the query.
 
         Args:
-            index_name (:obj:`str`): The index_name/identifier of the index.
-            queries (:obj:`Union[List[str], np.ndarray]`): The queries as a list of strings if `index_name` is a text index or as a numpy array if `index_name` is a vector index.
-            k (:obj:`int`): The number of examples to retrieve per query.
+            index_name (`str`):
+                The `index_name`/identifier of the index.
+            queries (`Union[List[str], np.ndarray]`):
+                The queries as a list of strings if `index_name` is a text index or as a numpy array if `index_name` is a vector index.
+            k (`int`):
+                The number of examples to retrieve per query.
 
         Returns:
-            total_scores (:obj:`List[List[float]`): The retrieval scores of the retrieved examples per query.
-            total_indices (:obj:`List[List[int]]`): The indices of the retrieved examples per query.
+            - total_scores (`List[List[float]`): The retrieval scores of the retrieved examples per query.
+            - total_indices (`List[List[int]]`): The indices of the retrieved examples per query.
         """
         self._check_index_is_initialized(index_name)
         return self._indexes[index_name].search_batch(queries, k)
@@ -697,13 +709,16 @@ class IndexableMixin:
         """Find the nearest examples in the dataset to the query.
 
         Args:
-            index_name (:obj:`str`): The index_name/identifier of the index.
-            query (:obj:`Union[str, np.ndarray]`): The query as a string if `index_name` is a text index or as a numpy array if `index_name` is a vector index.
-            k (:obj:`int`): The number of examples to retrieve.
+            index_name (`str`):
+                The index_name/identifier of the index.
+            query (`Union[str, np.ndarray]`):
+                The query as a string if `index_name` is a text index or as a numpy array if `index_name` is a vector index.
+            k (`int`):
+                The number of examples to retrieve.
 
         Returns:
-            scores (:obj:`List[float]`): The retrieval scores of the retrieved examples.
-            examples (:obj:`dict`): The retrieved examples.
+            - scores (`List[float]`): The retrieval scores of the retrieved examples.
+            - examples (`dict`): The retrieved examples.
         """
         self._check_index_is_initialized(index_name)
         scores, indices = self.search(index_name, query, k)
@@ -716,13 +731,16 @@ class IndexableMixin:
         """Find the nearest examples in the dataset to the query.
 
         Args:
-            index_name (:obj:`str`): The index_name/identifier of the index.
-            queries (:obj:`Union[List[str], np.ndarray]`): The queries as a list of strings if `index_name` is a text index or as a numpy array if `index_name` is a vector index.
-            k (:obj:`int`): The number of examples to retrieve per query.
+            index_name (`str`):
+                The `index_name`/identifier of the index.
+            queries (`Union[List[str], np.ndarray]`):
+                The queries as a list of strings if `index_name` is a text index or as a numpy array if `index_name` is a vector index.
+            k (`int`):
+                The number of examples to retrieve per query.
 
         Returns:
-            total_scores (`List[List[float]`): The retrieval scores of the retrieved examples per query.
-            total_examples (`List[dict]`): The retrieved examples per query.
+            - total_scores (`List[List[float]`): The retrieval scores of the retrieved examples per query.
+            - total_examples (`List[dict]`): The retrieved examples per query.
         """
         self._check_index_is_initialized(index_name)
         total_scores, total_indices = self.search_batch(index_name, queries, k)
