@@ -18,13 +18,15 @@
 
 import abc
 import collections
+import copy
+import dataclasses
 import re
-from dataclasses import dataclass, field
+from dataclasses import dataclass
 from typing import Dict, List, Optional, Union
 
 from .arrow_reader import FileInstructions, make_file_instructions
 from .naming import _split_re
-from .utils.py_utils import NonMutableDict
+from .utils.py_utils import NonMutableDict, asdict
 
 
 @dataclass
@@ -32,7 +34,15 @@ class SplitInfo:
     name: str = ""
     num_bytes: int = 0
     num_examples: int = 0
-    dataset_name: Optional[str] = None
+    shard_lengths: Optional[List[int]] = None
+
+    # Deprecated
+    # For backward compatibility, this field needs to always be included in files like
+    # dataset_infos.json and dataset_info.json files
+    # To do so, we always include it in the output of datasets.utils.py_utils.asdict(split_info)
+    dataset_name: Optional[str] = dataclasses.field(
+        default=None, metadata={"include_in_asdict_even_if_is_default": True}
+    )
 
     @property
     def file_instructions(self):
@@ -74,7 +84,7 @@ class SplitBase(metaclass=abc.ABCMeta):
     """Abstract base class for Split compositionality.
 
     See the
-    [guide on splits](/docs/datasets/loading#slice-splits)
+    [guide on splits](./loading#slice-splits)
     for more information.
 
     There are three parts to the composition:
@@ -252,7 +262,7 @@ class PercentSlice(metaclass=PercentSliceMeta):
     """Syntactic sugar for defining slice subsplits: `datasets.percent[75:-5]`.
 
     See the
-    [guide on splits](/docs/datasets/loading#slice-splits)
+    [guide on splits](./loading#slice-splits)
     for more information.
     """
     # pylint: enable=line-too-long
@@ -303,39 +313,44 @@ class _SubSplit(SplitBase):
 class NamedSplit(SplitBase):
     """Descriptor corresponding to a named split (train, test, ...).
 
-    Example::
-        Each descriptor can be composed with other using addition or slice. Ex::
+    Example:
+        Each descriptor can be composed with other using addition or slice:
 
+            ```py
             split = datasets.Split.TRAIN.subsplit(datasets.percent[0:25]) + datasets.Split.TEST
+            ```
 
         The resulting split will correspond to 25% of the train split merged with
         100% of the test split.
 
-    Warning:
-        A split cannot be added twice, so the following will fail::
+        A split cannot be added twice, so the following will fail:
 
+            ```py
             split = (
                     datasets.Split.TRAIN.subsplit(datasets.percent[:25]) +
                     datasets.Split.TRAIN.subsplit(datasets.percent[75:])
             )  # Error
             split = datasets.Split.TEST + datasets.Split.ALL  # Error
+            ```
 
-    Warning:
-        The slices can be applied only one time. So the following are valid::
+        The slices can be applied only one time. So the following are valid:
 
+            ```py
             split = (
                     datasets.Split.TRAIN.subsplit(datasets.percent[:25]) +
                     datasets.Split.TEST.subsplit(datasets.percent[:50])
             )
             split = (datasets.Split.TRAIN + datasets.Split.TEST).subsplit(datasets.percent[:50])
+            ```
 
+        But this is not valid:
 
-        But not::
-
+            ```py
             train = datasets.Split.TRAIN
             test = datasets.Split.TEST
             split = train.subsplit(datasets.percent[:25]).subsplit(datasets.percent[:25])
             split = (train.subsplit(datasets.percent[:25]) + test).subsplit(datasets.percent[:50])
+            ```
     """
 
     def __init__(self, name):
@@ -402,9 +417,9 @@ class Split:
       you do not want to use this during model iteration as you may overfit to it.
     - `ALL`: the union of all defined dataset splits.
 
-    Note: All splits, including compositions inherit from `datasets.SplitBase`
+    All splits, including compositions inherit from `datasets.SplitBase`.
 
-    See the :doc:`guide on splits </loading>` for more information.
+    See the [guide](./load_hub#splits) on splits for more information.
 
     Example:
 
@@ -496,7 +511,7 @@ class SplitReadInstruction:
         return split_instruction
 
     def get_list_sliced_split_info(self):
-        return list(sorted(self._splits.values(), key=lambda x: x.split_info.name))
+        return list(self._splits.values())
 
 
 class SplitDict(dict):
@@ -545,7 +560,7 @@ class SplitDict(dict):
             split_infos = list(split_infos.values())
 
         if dataset_name is None:
-            dataset_name = split_infos[0]["dataset_name"] if split_infos else None
+            dataset_name = split_infos[0].get("dataset_name") if split_infos else None
 
         split_dict = cls(dataset_name=dataset_name)
 
@@ -558,26 +573,46 @@ class SplitDict(dict):
 
     def to_split_dict(self):
         """Returns a list of SplitInfo protos that we have."""
-        # Return the SplitInfo, sorted by name
-        return sorted((s for s in self.values()), key=lambda s: s.name)
+        out = []
+        for split_name, split_info in self.items():
+            split_info = copy.deepcopy(split_info)
+            split_info.name = split_name
+            out.append(split_info)
+        return out
 
     def copy(self):
         return SplitDict.from_split_dict(self.to_split_dict(), self.dataset_name)
 
+    def _to_yaml_list(self) -> list:
+        out = [asdict(s) for s in self.to_split_dict()]
+        # we don't need the shard lengths in YAML, since it depends on max_shard_size and num_proc
+        for split_info_dict in out:
+            split_info_dict.pop("shard_lengths", None)
+        # we don't need the dataset_name attribute that is deprecated
+        for split_info_dict in out:
+            split_info_dict.pop("dataset_name", None)
+        return out
+
+    @classmethod
+    def _from_yaml_list(cls, yaml_data: list) -> "SplitDict":
+        return cls.from_split_dict(yaml_data)
+
 
 @dataclass
 class SplitGenerator:
     """Defines the split information for the generator.
 
     This should be used as returned value of
-    :meth:`GeneratorBasedBuilder._split_generators`.
-    See :meth:`GeneratorBasedBuilder._split_generators` for more info and example
+    `GeneratorBasedBuilder._split_generators`.
+    See `GeneratorBasedBuilder._split_generators` for more info and example
     of usage.
 
     Args:
-        name (str): Name of the Split for which the generator will
+        name (`str`):
+            Name of the `Split` for which the generator will
             create the examples.
-        **gen_kwargs: Keyword arguments to forward to the :meth:`DatasetBuilder._generate_examples` method
+        **gen_kwargs (additional keyword arguments):
+            Keyword arguments to forward to the `DatasetBuilder._generate_examples` method
             of the builder.
 
     Example:
@@ -591,8 +626,8 @@ class SplitGenerator:
     """
 
     name: str
-    gen_kwargs: Dict = field(default_factory=dict)
-    split_info: SplitInfo = field(init=False)
+    gen_kwargs: Dict = dataclasses.field(default_factory=dict)
+    split_info: SplitInfo = dataclasses.field(init=False)
 
     def __post_init__(self):
         self.name = str(self.name)  # Make sure we convert NamedSplits in strings
