@@ -1,4 +1,3 @@
-# coding=utf-8
 # Copyright 2020 The HuggingFace Datasets Authors and the TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,26 +16,12 @@
 # Lint as: python3
 """Util import."""
 
-from . import logging
-from .download_manager import DownloadManager, GenerateMode
-from .file_utils import DownloadConfig, cached_path, hf_bucket_url, is_remote_url, relative_to_absolute_path, temp_seed
-from .mock_download_manager import MockDownloadManager
-from .py_utils import (
-    NonMutableDict,
-    classproperty,
-    copyfunc,
-    dumps,
-    first_non_null_value,
-    flatten_nest_dict,
-    has_sufficient_disk_space,
-    map_nested,
-    memoize,
-    no_op_if_value_is_null,
-    size_str,
-    temporary_assignment,
-    unique_values,
-    zip_dict,
-    zip_nested,
-)
-from .tqdm_utils import disable_progress_bar, is_progress_bar_enabled, set_progress_bar_enabled, tqdm
+__all__ = [
+    "Version",
+    "disable_progress_bar",
+    "enable_progress_bar",
+    "is_progress_bar_enabled",
+]
+
+from .logging import disable_progress_bar, enable_progress_bar, is_progress_bar_enabled
 from .version import Version
