@@ -17,25 +17,27 @@
 
 """
 
-import contextlib
 import copy
 import functools
 import itertools
+import multiprocessing.pool
 import os
-import pickle
+import queue
 import re
-import sys
 import types
 from contextlib import contextmanager
 from dataclasses import fields, is_dataclass
 from io import BytesIO as StringIO
-from multiprocessing import Pool, RLock
+from multiprocessing import Manager, Pool, RLock
+from queue import Empty
 from shutil import disk_usage
 from types import CodeType, FunctionType
-from typing import Callable, ClassVar, Dict, Generic, List, Optional, Tuple, Union
+from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, TypeVar, Union
 from urllib.parse import urlparse
 
 import dill
+import multiprocess
+import multiprocess.pool
 import numpy as np
 from packaging import version
 from tqdm.auto import tqdm
@@ -88,7 +90,7 @@ def size_str(size_in_bytes):
 
 def convert_file_size_to_int(size: Union[int, str]) -> int:
     """
-    Converts a size expressed as a string with digits an unit (like `"5MB"`) to an integer (in bytes).
+    Converts a size expressed as a string with digits an unit (like `"50MB"`) to an integer (in bytes).
 
     Args:
         size (`int` or `str`): The size to convert. Will be directly returned if an `int`.
@@ -102,12 +104,22 @@ def convert_file_size_to_int(size: Union[int, str]) -> int:
     """
     if isinstance(size, int):
         return size
+    if size.upper().endswith("PIB"):
+        return int(size[:-3]) * (2**50)
+    if size.upper().endswith("TIB"):
+        return int(size[:-3]) * (2**40)
     if size.upper().endswith("GIB"):
         return int(size[:-3]) * (2**30)
     if size.upper().endswith("MIB"):
         return int(size[:-3]) * (2**20)
     if size.upper().endswith("KIB"):
         return int(size[:-3]) * (2**10)
+    if size.upper().endswith("PB"):
+        int_size = int(size[:-2]) * (10**15)
+        return int_size // 8 if size.endswith("b") else int_size
+    if size.upper().endswith("TB"):
+        int_size = int(size[:-2]) * (10**12)
+        return int_size // 8 if size.endswith("b") else int_size
     if size.upper().endswith("GB"):
         int_size = int(size[:-2]) * (10**9)
         return int_size // 8 if size.endswith("b") else int_size
@@ -117,7 +129,7 @@ def convert_file_size_to_int(size: Union[int, str]) -> int:
     if size.upper().endswith("KB"):
         int_size = int(size[:-2]) * (10**3)
         return int_size // 8 if size.endswith("b") else int_size
-    raise ValueError("`size` is not in a valid format. Use an integer followed by the unit, e.g., '5GB'.")
+    raise ValueError(f"`size={size}` is not in a valid format. Use an integer followed by the unit, e.g., '5GB'.")
 
 
 def string_to_dict(string: str, pattern: str) -> Dict[str, str]:
@@ -154,7 +166,10 @@ def string_to_dict(string: str, pattern: str) -> Dict[str, str]:
 
 
 def asdict(obj):
-    """Convert an object to its dictionary representation recursively."""
+    """Convert an object to its dictionary representation recursively.
+
+    <Added version="2.4.0"/>
+    """
 
     # Implementation based on https://docs.python.org/3/library/dataclasses.html#dataclasses.asdict
 
@@ -167,7 +182,8 @@ def asdict(obj):
             result = {}
             for f in fields(obj):
                 value = _asdict_inner(getattr(obj, f.name))
-                result[f.name] = value
+                if not f.init or value != f.default or f.metadata.get("include_in_asdict_even_if_is_default", False):
+                    result[f.name] = value
             return result
         elif isinstance(obj, tuple) and hasattr(obj, "_fields"):
             # obj is a namedtuple
@@ -188,7 +204,7 @@ def asdict(obj):
     return _asdict_inner(obj)
 
 
-@contextlib.contextmanager
+@contextmanager
 def temporary_assignment(obj, attr, value):
     """Temporarily assign obj.attr to value."""
     original = getattr(obj, attr, None)
@@ -355,19 +371,54 @@ def _single_map_nested(args):
 
 
 def map_nested(
-    function,
-    data_struct,
+    function: Callable[[Any], Any],
+    data_struct: Any,
     dict_only: bool = False,
     map_list: bool = True,
     map_tuple: bool = False,
     map_numpy: bool = False,
     num_proc: Optional[int] = None,
-    types=None,
+    parallel_min_length: int = 2,
+    types: Optional[tuple] = None,
     disable_tqdm: bool = True,
     desc: Optional[str] = None,
-):
+) -> Any:
     """Apply a function recursively to each element of a nested data struct.
-    If num_proc > 1 and the length of data_struct is longer than num_proc: use multi-processing
+
+    Use multiprocessing if num_proc > 1 and the length of data_struct is greater than or equal to
+    `parallel_min_length`.
+
+    <Changed version="2.5.0">
+
+    Before version 2.5.0, multiprocessing was not used if `num_proc` was greater than or equal to ``len(iterable)``.
+
+    Now, if `num_proc` is greater than or equal to ``len(iterable)``, `num_proc` is set to ``len(iterable)`` and
+    multiprocessing is used.
+
+    </Changed>
+
+    Args:
+        function (`Callable`): Function to be applied to `data_struct`.
+        data_struct (`Any`): Data structure to apply `function` to.
+        dict_only (`bool`, default `False`): Whether only apply `function` recursively to `dict` values in
+            `data_struct`.
+        map_list (`bool`, default `True`): Whether also apply `function` recursively to `list` elements (besides `dict`
+            values).
+        map_tuple (`bool`, default `False`): Whether also apply `function` recursively to `tuple` elements (besides
+            `dict` values).
+        map_numpy (`bool, default `False`): Whether also apply `function` recursively to `numpy.array` elements (besides
+            `dict` values).
+        num_proc (`int`, *optional*): Number of processes.
+        parallel_min_length (`int`, default `2`): Minimum length of `data_struct` required for parallel
+            processing.
+            <Added version="2.5.0"/>
+        types (`tuple`, *optional*): Additional types (besides `dict` values) to apply `function` recursively to their
+            elements.
+        disable_tqdm (`bool`, default `True`): Whether to disable the tqdm progressbar.
+        desc (`str`, *optional*): Prefix for the tqdm progressbar.
+
+    Returns:
+        `Any`
     """
     if types is None:
         types = []
@@ -389,12 +440,13 @@ def map_nested(
 
     if num_proc is None:
         num_proc = 1
-    if num_proc <= 1 or len(iterable) <= num_proc:
+    if num_proc <= 1 or len(iterable) < parallel_min_length:
         mapped = [
             _single_map_nested((function, obj, types, None, True, None))
             for obj in logging.tqdm(iterable, disable=disable_tqdm, desc=desc)
         ]
     else:
+        num_proc = num_proc if num_proc <= len(iterable) else len(iterable)
         split_kwds = []  # We organize the splits ourselve (contiguous splits)
         for index in range(num_proc):
             div = len(iterable) // num_proc
@@ -552,18 +604,77 @@ class Pickler(dill.Pickler):
 
     dispatch = dill._dill.MetaCatchingDict(dill.Pickler.dispatch.copy())
 
-    def save_global(self, obj, name=None):
-        if sys.version_info[:2] < (3, 7) and _CloudPickleTypeHintFix._is_parametrized_type_hint(
-            obj
-        ):  # noqa  # pragma: no branch
-            # Parametrized typing constructs in Python < 3.7 are not compatible
-            # with type checks and ``isinstance`` semantics. For this reason,
-            # it is easier to detect them using a duck-typing-based check
-            # (``_is_parametrized_type_hint``) than to populate the Pickler's
-            # dispatch with type-specific savers.
-            _CloudPickleTypeHintFix._save_parametrized_type_hint(self, obj)
-        else:
-            dill.Pickler.save_global(self, obj, name=name)
+    def save(self, obj, save_persistent_id=True):
+        # lazy registration of reduction functions
+        obj_type = type(obj)
+        if obj_type not in Pickler.dispatch:
+            if config.DILL_VERSION < version.parse("0.3.6"):
+
+                def dill_log(pickler, msg):
+                    dill._dill.log.info(msg)
+
+            elif config.DILL_VERSION.release[:3] == version.parse("0.3.6").release:
+
+                def dill_log(pickler, msg):
+                    dill._dill.logger.trace(pickler, msg)
+
+            if (obj_type.__module__, obj_type.__name__) == ("_regex", "Pattern"):
+                try:
+                    import regex
+
+                    @pklregister(obj_type)
+                    def _save_regex(pickler, obj):
+                        dill_log(pickler, f"Re: {obj}")
+                        args = (
+                            obj.pattern,
+                            obj.flags,
+                        )
+                        pickler.save_reduce(regex.compile, args, obj=obj)
+                        dill_log(pickler, "# Re")
+                        return
+
+                except ImportError:
+                    pass
+            elif (obj_type.__module__, obj_type.__name__) == ("torch", "Tensor"):
+                try:
+                    import torch
+
+                    @pklregister(obj_type)
+                    def _save_tensor(pickler, obj):
+                        dill_log(pickler, f"To: {obj}")
+                        if obj.requires_grad:
+                            args = (obj.detach().cpu().numpy(),)
+                        else:
+                            args = (obj.cpu().numpy(),)
+                        pickler.save_reduce(torch.from_numpy, args, obj=obj)
+                        dill_log(pickler, "# To")
+                        return
+
+                except ImportError:
+                    pass
+            elif obj_type.__module__.startswith("spacy.lang") and any(
+                (cls.__module__, cls.__name__) == ("spacy.language", "Language") for cls in obj_type.__mro__
+            ):
+                try:
+                    import spacy
+
+                    @pklregister(obj_type)
+                    def _save_lang(pickler, obj):
+                        def _create_lang(config, bytes_data):
+                            lang_cls = spacy.util.get_lang_class(config["nlp"]["lang"])
+                            nlp = lang_cls.from_config(config)
+                            return nlp.from_bytes(bytes_data)
+
+                        dill_log(pickler, f"Sp: {obj}")
+                        args = (obj.config, obj.to_bytes())
+                        pickler.save_reduce(_create_lang, args, obj=obj)
+                        dill_log(pickler, "# Sp")
+                        return
+
+                except ImportError:
+                    pass
+
+        dill.Pickler.save(self, obj, save_persistent_id=save_persistent_id)
 
     def memoize(self, obj):
         # don't memoize strings since two identical strings can have different python ids
@@ -577,7 +688,7 @@ def dump(obj, file):
     return
 
 
-@contextlib.contextmanager
+@contextmanager
 def _no_cache_fields(obj):
     try:
         if (
@@ -610,76 +721,211 @@ def pklregister(t):
     return proxy
 
 
-class _CloudPickleTypeHintFix:
-    """
-    Type hints can't be properly pickled in python < 3.7
-    CloudPickle provided a way to make it work in older versions.
-    This class provide utilities to fix pickling of type hints in older versions.
-    from https://github.com/cloudpipe/cloudpickle/pull/318/files
-    """
+if config.DILL_VERSION < version.parse("0.3.6"):
 
-    def _is_parametrized_type_hint(obj):
-        # This is very cheap but might generate false positives.
-        origin = getattr(obj, "__origin__", None)  # typing Constructs
-        values = getattr(obj, "__values__", None)  # typing_extensions.Literal
-        type_ = getattr(obj, "__type__", None)  # typing_extensions.Final
-        return origin is not None or values is not None or type_ is not None
-
-    def _create_parametrized_type_hint(origin, args):
-        return origin[args]
-
-    def _save_parametrized_type_hint(pickler, obj):
-        # The distorted type check sematic for typing construct becomes:
-        # ``type(obj) is type(TypeHint)``, which means "obj is a
-        # parametrized TypeHint"
-        if type(obj) is type(Literal):  # pragma: no branch
-            initargs = (Literal, obj.__values__)
-        elif type(obj) is type(Final):  # pragma: no branch
-            initargs = (Final, obj.__type__)
-        elif type(obj) is type(ClassVar):
-            initargs = (ClassVar, obj.__type__)
-        elif type(obj) in [type(Union), type(Tuple), type(Generic)]:
-            initargs = (obj.__origin__, obj.__args__)
-        elif type(obj) is type(Callable):
-            args = obj.__args__
-            if args[0] is Ellipsis:
-                initargs = (obj.__origin__, args)
+    @pklregister(CodeType)
+    def _save_code(pickler, obj):
+        """
+        From dill._dill.save_code
+        This is a modified version that removes the origin (filename + line no.)
+        of functions created in notebooks or shells for example.
+        """
+        dill._dill.log.info(f"Co: {obj}")
+        # The filename of a function is the .py file where it is defined.
+        # Filenames of functions created in notebooks or shells start with '<'
+        # ex: <ipython-input-13-9ed2afe61d25> for ipython, and <stdin> for shell
+        # Filenames of functions created in ipykernel the filename
+        # look like f"{tempdir}/ipykernel_{id1}/{id2}.py"
+        # Moreover lambda functions have a special name: '<lambda>'
+        # ex: (lambda x: x).__code__.co_name == "<lambda>"  # True
+        #
+        # For the hashing mechanism we ignore where the function has been defined
+        # More specifically:
+        # - we ignore the filename of special functions (filename starts with '<')
+        # - we always ignore the line number
+        # - we only use the base name of the file instead of the whole path,
+        # to be robust in case a script is moved for example.
+        #
+        # Only those two lines are different from the original implementation:
+        co_filename = (
+            ""
+            if obj.co_filename.startswith("<")
+            or (
+                len(obj.co_filename.split(os.path.sep)) > 1
+                and obj.co_filename.split(os.path.sep)[-2].startswith("ipykernel_")
+            )
+            or obj.co_name == "<lambda>"
+            else os.path.basename(obj.co_filename)
+        )
+        co_firstlineno = 1
+        # The rest is the same as in the original dill implementation
+        if dill._dill.PY3:
+            if hasattr(obj, "co_posonlyargcount"):
+                args = (
+                    obj.co_argcount,
+                    obj.co_posonlyargcount,
+                    obj.co_kwonlyargcount,
+                    obj.co_nlocals,
+                    obj.co_stacksize,
+                    obj.co_flags,
+                    obj.co_code,
+                    obj.co_consts,
+                    obj.co_names,
+                    obj.co_varnames,
+                    co_filename,
+                    obj.co_name,
+                    co_firstlineno,
+                    obj.co_lnotab,
+                    obj.co_freevars,
+                    obj.co_cellvars,
+                )
             else:
-                initargs = (obj.__origin__, (list(args[:-1]), args[-1]))
-        else:  # pragma: no cover
-            raise pickle.PicklingError(f"Datasets pickle Error: Unknown type {type(obj)}")
-        pickler.save_reduce(_CloudPickleTypeHintFix._create_parametrized_type_hint, initargs, obj=obj)
+                args = (
+                    obj.co_argcount,
+                    obj.co_kwonlyargcount,
+                    obj.co_nlocals,
+                    obj.co_stacksize,
+                    obj.co_flags,
+                    obj.co_code,
+                    obj.co_consts,
+                    obj.co_names,
+                    obj.co_varnames,
+                    co_filename,
+                    obj.co_name,
+                    co_firstlineno,
+                    obj.co_lnotab,
+                    obj.co_freevars,
+                    obj.co_cellvars,
+                )
+        else:
+            args = (
+                obj.co_argcount,
+                obj.co_nlocals,
+                obj.co_stacksize,
+                obj.co_flags,
+                obj.co_code,
+                obj.co_consts,
+                obj.co_names,
+                obj.co_varnames,
+                co_filename,
+                obj.co_name,
+                co_firstlineno,
+                obj.co_lnotab,
+                obj.co_freevars,
+                obj.co_cellvars,
+            )
+        pickler.save_reduce(CodeType, args, obj=obj)
+        dill._dill.log.info("# Co")
+        return
 
+elif config.DILL_VERSION.release[:3] == version.parse("0.3.6").release:
 
-@pklregister(CodeType)
-def _save_code(pickler, obj):
-    """
-    From dill._dill.save_code
-    This is a modified version that removes the origin (filename + line no.)
-    of functions created in notebooks or shells for example.
-    """
-    dill._dill.log.info(f"Co: {obj}")
-    # The filename of a function is the .py file where it is defined.
-    # Filenames of functions created in notebooks or shells start with '<'
-    # ex: <ipython-input-13-9ed2afe61d25> for ipython, and <stdin> for shell
-    # Moreover lambda functions have a special name: '<lambda>'
-    # ex: (lambda x: x).__code__.co_name == "<lambda>"  # True
-    #
-    # For the hashing mechanism we ignore where the function has been defined
-    # More specifically:
-    # - we ignore the filename of special functions (filename starts with '<')
-    # - we always ignore the line number
-    # - we only use the base name of the file instead of the whole path,
-    # to be robust in case a script is moved for example.
-    #
-    # Only those two lines are different from the original implementation:
-    co_filename = (
-        "" if obj.co_filename.startswith("<") or obj.co_name == "<lambda>" else os.path.basename(obj.co_filename)
-    )
-    co_firstlineno = 1
-    # The rest is the same as in the original dill implementation
-    if dill._dill.PY3:
-        if hasattr(obj, "co_posonlyargcount"):
+    # From: https://github.com/uqfoundation/dill/blob/dill-0.3.6/dill/_dill.py#L1104
+    @pklregister(CodeType)
+    def save_code(pickler, obj):
+        dill._dill.logger.trace(pickler, "Co: %s", obj)
+
+        ############################################################################################################
+        # Modification here for huggingface/datasets
+        # The filename of a function is the .py file where it is defined.
+        # Filenames of functions created in notebooks or shells start with '<'
+        # ex: <ipython-input-13-9ed2afe61d25> for ipython, and <stdin> for shell
+        # Filenames of functions created in ipykernel the filename
+        # look like f"{tempdir}/ipykernel_{id1}/{id2}.py"
+        # Moreover lambda functions have a special name: '<lambda>'
+        # ex: (lambda x: x).__code__.co_name == "<lambda>"  # True
+        #
+        # For the hashing mechanism we ignore where the function has been defined
+        # More specifically:
+        # - we ignore the filename of special functions (filename starts with '<')
+        # - we always ignore the line number
+        # - we only use the base name of the file instead of the whole path,
+        # to be robust in case a script is moved for example.
+        #
+        # Only those two lines are different from the original implementation:
+        co_filename = (
+            ""
+            if obj.co_filename.startswith("<")
+            or (
+                len(obj.co_filename.split(os.path.sep)) > 1
+                and obj.co_filename.split(os.path.sep)[-2].startswith("ipykernel_")
+            )
+            or obj.co_name == "<lambda>"
+            else os.path.basename(obj.co_filename)
+        )
+        co_firstlineno = 1
+        # The rest is the same as in the original dill implementation, except for the replacements:
+        # - obj.co_filename => co_filename
+        # - obj.co_firstlineno => co_firstlineno
+        ############################################################################################################
+
+        if hasattr(obj, "co_endlinetable"):  # python 3.11a (20 args)
+            args = (
+                obj.co_lnotab,  # for < python 3.10 [not counted in args]
+                obj.co_argcount,
+                obj.co_posonlyargcount,
+                obj.co_kwonlyargcount,
+                obj.co_nlocals,
+                obj.co_stacksize,
+                obj.co_flags,
+                obj.co_code,
+                obj.co_consts,
+                obj.co_names,
+                obj.co_varnames,
+                co_filename,  # Modification for huggingface/datasets ############################################
+                obj.co_name,
+                obj.co_qualname,
+                co_firstlineno,  # Modification for huggingface/datasets #########################################
+                obj.co_linetable,
+                obj.co_endlinetable,
+                obj.co_columntable,
+                obj.co_exceptiontable,
+                obj.co_freevars,
+                obj.co_cellvars,
+            )
+        elif hasattr(obj, "co_exceptiontable"):  # python 3.11 (18 args)
+            args = (
+                obj.co_lnotab,  # for < python 3.10 [not counted in args]
+                obj.co_argcount,
+                obj.co_posonlyargcount,
+                obj.co_kwonlyargcount,
+                obj.co_nlocals,
+                obj.co_stacksize,
+                obj.co_flags,
+                obj.co_code,
+                obj.co_consts,
+                obj.co_names,
+                obj.co_varnames,
+                co_filename,  # Modification for huggingface/datasets ############################################
+                obj.co_name,
+                obj.co_qualname,
+                co_firstlineno,  # Modification for huggingface/datasets #########################################
+                obj.co_linetable,
+                obj.co_exceptiontable,
+                obj.co_freevars,
+                obj.co_cellvars,
+            )
+        elif hasattr(obj, "co_linetable"):  # python 3.10 (16 args)
+            args = (
+                obj.co_lnotab,  # for < python 3.10 [not counted in args]
+                obj.co_argcount,
+                obj.co_posonlyargcount,
+                obj.co_kwonlyargcount,
+                obj.co_nlocals,
+                obj.co_stacksize,
+                obj.co_flags,
+                obj.co_code,
+                obj.co_consts,
+                obj.co_names,
+                obj.co_varnames,
+                co_filename,  # Modification for huggingface/datasets ############################################
+                obj.co_name,
+                co_firstlineno,  # Modification for huggingface/datasets #########################################
+                obj.co_linetable,
+                obj.co_freevars,
+                obj.co_cellvars,
+            )
+        elif hasattr(obj, "co_posonlyargcount"):  # python 3.8 (16 args)
             args = (
                 obj.co_argcount,
                 obj.co_posonlyargcount,
@@ -691,14 +937,14 @@ def _save_code(pickler, obj):
                 obj.co_consts,
                 obj.co_names,
                 obj.co_varnames,
-                co_filename,
+                co_filename,  # Modification for huggingface/datasets ############################################
                 obj.co_name,
-                co_firstlineno,
+                co_firstlineno,  # Modification for huggingface/datasets #########################################
                 obj.co_lnotab,
                 obj.co_freevars,
                 obj.co_cellvars,
             )
-        else:
+        else:  # python 3.7 (15 args)
             args = (
                 obj.co_argcount,
                 obj.co_kwonlyargcount,
@@ -709,33 +955,17 @@ def _save_code(pickler, obj):
                 obj.co_consts,
                 obj.co_names,
                 obj.co_varnames,
-                co_filename,
+                co_filename,  # Modification for huggingface/datasets ############################################
                 obj.co_name,
-                co_firstlineno,
+                co_firstlineno,  # Modification for huggingface/datasets #########################################
                 obj.co_lnotab,
                 obj.co_freevars,
                 obj.co_cellvars,
             )
-    else:
-        args = (
-            obj.co_argcount,
-            obj.co_nlocals,
-            obj.co_stacksize,
-            obj.co_flags,
-            obj.co_code,
-            obj.co_consts,
-            obj.co_names,
-            obj.co_varnames,
-            co_filename,
-            obj.co_name,
-            co_firstlineno,
-            obj.co_lnotab,
-            obj.co_freevars,
-            obj.co_cellvars,
-        )
-    pickler.save_reduce(CodeType, args, obj=obj)
-    dill._dill.log.info("# Co")
-    return
+
+        pickler.save_reduce(dill._dill._create_code, args, obj=obj)
+        dill._dill.logger.trace(pickler, "# Co")
+        return
 
 
 if config.DILL_VERSION < version.parse("0.3.5"):
@@ -813,7 +1043,7 @@ if config.DILL_VERSION < version.parse("0.3.5"):
             dill._dill.log.info("# F2")
         return
 
-else:  # config.DILL_VERSION >= version.parse("0.3.5")
+elif config.DILL_VERSION.release[:3] == version.parse("0.3.5").release:  # 0.3.5, 0.3.5.1
 
     # https://github.com/uqfoundation/dill/blob/dill-0.3.5.1/dill/_dill.py
     @pklregister(FunctionType)
@@ -821,7 +1051,6 @@ else:  # config.DILL_VERSION >= version.parse("0.3.5")
         if not dill._dill._locate_function(obj, pickler):
             dill._dill.log.info("F1: %s" % obj)
             _recurse = getattr(pickler, "_recurse", None)
-            # _byref = getattr(pickler, "_byref", None)  # TODO: not used
             _postproc = getattr(pickler, "_postproc", None)
             _main_modified = getattr(pickler, "_main_modified", None)
             _original_main = getattr(pickler, "_original_main", dill._dill.__builtin__)  # 'None'
@@ -958,6 +1187,147 @@ else:  # config.DILL_VERSION >= version.parse("0.3.5")
             dill._dill.log.info("# F2")
         return
 
+elif config.DILL_VERSION.release[:3] == version.parse("0.3.6").release:
+
+    # From: https://github.com/uqfoundation/dill/blob/dill-0.3.6/dill/_dill.py#L1739
+    @pklregister(FunctionType)
+    def save_function(pickler, obj):
+        if not dill._dill._locate_function(obj, pickler):
+            if type(obj.__code__) is not CodeType:
+                # Some PyPy builtin functions have no module name, and thus are not
+                # able to be located
+                module_name = getattr(obj, "__module__", None)
+                if module_name is None:
+                    module_name = dill._dill.__builtin__.__name__
+                module = dill._dill._import_module(module_name, safe=True)
+                _pypy_builtin = False
+                try:
+                    found, _ = dill._dill._getattribute(module, obj.__qualname__)
+                    if getattr(found, "__func__", None) is obj:
+                        _pypy_builtin = True
+                except AttributeError:
+                    pass
+
+                if _pypy_builtin:
+                    dill._dill.logger.trace(pickler, "F3: %s", obj)
+                    pickler.save_reduce(getattr, (found, "__func__"), obj=obj)
+                    dill._dill.logger.trace(pickler, "# F3")
+                    return
+
+            dill._dill.logger.trace(pickler, "F1: %s", obj)
+            _recurse = getattr(pickler, "_recurse", None)
+            _postproc = getattr(pickler, "_postproc", None)
+            _main_modified = getattr(pickler, "_main_modified", None)
+            _original_main = getattr(pickler, "_original_main", dill._dill.__builtin__)  # 'None'
+            postproc_list = []
+            if _recurse:
+                # recurse to get all globals referred to by obj
+                from dill.detect import globalvars
+
+                globs_copy = globalvars(obj, recurse=True, builtin=True)
+
+                # Add the name of the module to the globs dictionary to prevent
+                # the duplication of the dictionary. Pickle the unpopulated
+                # globals dictionary and set the remaining items after the function
+                # is created to correctly handle recursion.
+                globs = {"__name__": obj.__module__}
+            else:
+                globs_copy = obj.__globals__
+
+                # If the globals is the __dict__ from the module being saved as a
+                # session, substitute it by the dictionary being actually saved.
+                if _main_modified and globs_copy is _original_main.__dict__:
+                    globs_copy = getattr(pickler, "_main", _original_main).__dict__
+                    globs = globs_copy
+                # If the globals is a module __dict__, do not save it in the pickle.
+                elif (
+                    globs_copy is not None
+                    and obj.__module__ is not None
+                    and getattr(dill._dill._import_module(obj.__module__, True), "__dict__", None) is globs_copy
+                ):
+                    globs = globs_copy
+                else:
+                    globs = {"__name__": obj.__module__}
+
+            ########################################################################################################
+            # Modification here for huggingface/datasets
+            # - globs is a dictionary with keys = var names (str) and values = python objects
+            # - globs_copy is a dictionary with keys = var names (str) and values = ids of the python objects
+            # However the dictionary is not always loaded in the same order,
+            # therefore we have to sort the keys to make deterministic.
+            # This is important to make `dump` deterministic.
+            # Only these line are different from the original implementation:
+            # START
+            globs_is_globs_copy = globs is globs_copy
+            globs = dict(sorted(globs.items()))
+            if globs_is_globs_copy:
+                globs_copy = globs
+            elif globs_copy is not None:
+                globs_copy = dict(sorted(globs_copy.items()))
+            # END
+            ########################################################################################################
+
+            if globs_copy is not None and globs is not globs_copy:
+                # In the case that the globals are copied, we need to ensure that
+                # the globals dictionary is updated when all objects in the
+                # dictionary are already created.
+                glob_ids = {id(g) for g in globs_copy.values()}
+                for stack_element in _postproc:
+                    if stack_element in glob_ids:
+                        _postproc[stack_element].append((dill._dill._setitems, (globs, globs_copy)))
+                        break
+                else:
+                    postproc_list.append((dill._dill._setitems, (globs, globs_copy)))
+
+            closure = obj.__closure__
+            state_dict = {}
+            for fattrname in ("__doc__", "__kwdefaults__", "__annotations__"):
+                fattr = getattr(obj, fattrname, None)
+                if fattr is not None:
+                    state_dict[fattrname] = fattr
+            if obj.__qualname__ != obj.__name__:
+                state_dict["__qualname__"] = obj.__qualname__
+            if "__name__" not in globs or obj.__module__ != globs["__name__"]:
+                state_dict["__module__"] = obj.__module__
+
+            state = obj.__dict__
+            if type(state) is not dict:
+                state_dict["__dict__"] = state
+                state = None
+            if state_dict:
+                state = state, state_dict
+
+            dill._dill._save_with_postproc(
+                pickler,
+                (dill._dill._create_function, (obj.__code__, globs, obj.__name__, obj.__defaults__, closure), state),
+                obj=obj,
+                postproc_list=postproc_list,
+            )
+
+            # Lift closure cell update to earliest function (#458)
+            if _postproc:
+                topmost_postproc = next(iter(_postproc.values()), None)
+                if closure and topmost_postproc:
+                    for cell in closure:
+                        possible_postproc = (setattr, (cell, "cell_contents", obj))
+                        try:
+                            topmost_postproc.remove(possible_postproc)
+                        except ValueError:
+                            continue
+
+                        # Change the value of the cell
+                        pickler.save_reduce(*possible_postproc)
+                        # pop None created by calling preprocessing step off stack
+                        pickler.write(bytes("0", "UTF-8"))
+
+            dill._dill.logger.trace(pickler, "# F1")
+        else:
+            dill._dill.logger.trace(pickler, "F2: %s", obj)
+            name = getattr(obj, "__qualname__", getattr(obj, "__name__", None))
+            dill._dill.StockPickler.save_global(pickler, obj, name=name)
+            dill._dill.logger.trace(pickler, "# F2")
+        return
+
 
 def copyfunc(func):
     result = types.FunctionType(func.__code__, func.__globals__, func.__name__, func.__defaults__, func.__closure__)
@@ -965,19 +1335,32 @@ def copyfunc(func):
     return result
 
 
-try:
-    import regex
+Y = TypeVar("Y")
 
-    @pklregister(type(regex.Regex("", 0)))
-    def _save_regex(pickler, obj):
-        dill._dill.log.info(f"Re: {obj}")
-        args = (
-            obj.pattern,
-            obj.flags,
-        )
-        pickler.save_reduce(regex.compile, args, obj=obj)
-        dill._dill.log.info("# Re")
-        return
 
-except ImportError:
-    pass
+def _write_generator_to_queue(queue: queue.Queue, func: Callable[..., Iterable[Y]], kwargs: dict) -> int:
+    for i, result in enumerate(func(**kwargs)):
+        queue.put(result)
+    return i
+
+
+def iflatmap_unordered(
+    pool: Union[multiprocessing.pool.Pool, multiprocess.pool.Pool],
+    func: Callable[..., Iterable[Y]],
+    *,
+    kwargs_iterable: Iterable[dict],
+) -> Iterable[Y]:
+    manager_cls = Manager if isinstance(pool, multiprocessing.pool.Pool) else multiprocess.Manager
+    with manager_cls() as manager:
+        queue = manager.Queue()
+        async_results = [
+            pool.apply_async(_write_generator_to_queue, (queue, func, kwargs)) for kwargs in kwargs_iterable
+        ]
+        while True:
+            try:
+                yield queue.get(timeout=0.05)
+            except Empty:
+                if all(async_result.ready() for async_result in async_results) and queue.empty():
+                    break
+        # we get the result in case there's an error to raise
+        [async_result.get() for async_result in async_results]
